<?php

/**
 * Plugin Name:       Checkoutify – Checkout Field Editor For WooCommerce
 * Plugin URI:        https://getwpfunnels.com
 * Description:       Craft the ideal online shopping journey with the Advanced Drag and Drop Checkout Customizer. From fields to layouts, colors, typography, and order bumps, craft a personalized and simplified checkout experience for your buyers.
 * Requires at least: 5.8
 * Requires PHP:      7.4
 * Version:           1.0.10
 * Tested upto:       6.6.1
 * Author:            WPFunnels Team
 * Author URI:        https://getwpfunnels.com
 * License:           GPL-2.0-or-later
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       checkoutify
 *
 * @package Checkoutify
 */

use RexTheme\Checkoutify\Classes\Updater;
use RexTheme\Checkoutify\Common\Keys;
use RexTheme\Checkoutify\Dependencies\CheckDependency;

defined( 'ABSPATH' ) || exit;
/**
 * Activating the plugin.
 *
 * This method is called when the plugin is activated. It can be used to perform tasks such as creating necessary migrations and seeders.
 *
 * @since 1.0.0
 */
final class Checkoutify {

	/**
	* Plugin version.
	*
	* @var string
	*/

	const VERSION = '1.0.10';

	/**
	* Plugin slug.
	*
	* @var string
	*
	* @since 1.0.0
	*/

	const SLUG = 'checkoutify';

	/**
	 * Holds various class instances.
	 *
	 * @var array
	 *
	 * @since 1.0.0
	 */
	private $container = array();

	/**
	 * Constructor for the Checkoutify class.
	 *
	 * Sets up all the appropriate hooks and actions within our plugin.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {
		require_once __DIR__ . '/vendor/autoload.php';

		$this->define_constants();
		register_activation_hook( __FILE__, array( $this, 'activate' ) );
		register_deactivation_hook( __FILE__, array( $this, 'deactivate' ) );

		add_action( 'wp_loaded', array( $this, 'flush_rewrite_rules' ) );
		$this->init_plugin();
		$notice = __('Checkoutify requires WooCommerce for proper functionality. Please install WooCommerce before using Checkoutify.', 'checkoutify');
		//Check woocommerce dependency.
		new CheckDependency("woocommerce/woocommerce.php",$notice);
		add_action( 'plugins_loaded', array( $this, 'loadPlugin' ), 99 );
        add_action( 'activated_plugin', array( $this, 'rex_checkoutify_after_activation' ) );
	}

	/**
	 * Create wigdet instance
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function loadPlugin() {
		$this->container['widget'] = RexTheme\Checkoutify\Widgets\WidgetManager::getInstance()->init();
	}

	/**
	 * Initializes the PluginBoilerplate() class.
	 *
	 * Checks for an existing PluginBoilerplate() instance
	 * and if it doesn't find one, creates it.
	 *
	 * @since 1.0.0
	 *
	 * @return Checkoutify|bool
	 */
	public static function init() {
		static $instance = false;

		if ( ! $instance ) {
			$instance = new Checkoutify();
		}

		return $instance;
	}

	/**
	 * Magic getter method to retrieve properties or values dynamically.
	 *
	 * This method is invoked when an undefined or inaccessible property is accessed using the "get" syntax. It allows for dynamic retrieval of property values.
	 *
	 * @param string $prop The name of the property to retrieve.
	 *
	 * @return mixed|null The value of the property if it exists in the container, or null if the property is not found.
	 */
	public function __get( $prop ) {
		if ( array_key_exists( $prop, $this->container ) ) {
			return $this->container[ $prop ];
		}

		return $this->{$prop};
	}

	/**
	 * Magic isset method to check if a property is set or exists.
	 *
	 * This method is invoked when the "isset" function or empty function is used to determine if a property is set or exists.
	 *
	 * @param string $prop The name of the property to check.
	 *
	 * @return bool Returns true if the property is set in the container or the object, or false if it is not set.
	 */
	public function __isset( $prop ) {
		return isset( $this->{$prop} ) || isset( $this->container[ $prop ] );
	}

	/**
	 * Define the constants.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function define_constants() {
		define( 'CHECKOUTIFY_VERSION', self::VERSION );
		define( 'CHECKOUTIFY_SLUG', self::SLUG );
		define( 'CHECKOUTIFY_FILE', __FILE__ );
		define( 'CHECKOUTIFY_DIR', __DIR__ );
		define( 'CHECKOUTIFY_PATH', dirname( CHECKOUTIFY_FILE ) );
		define( 'CHECKOUTIFY_INCLUDES', CHECKOUTIFY_PATH . '/includes' );
		define( 'CHECKOUTIFY_TEMPLATE_PATH', CHECKOUTIFY_PATH . '/views' );
		define( 'CHECKOUTIFY_URL', plugins_url( '', CHECKOUTIFY_FILE ) );
		define( 'CHECKOUTIFY_BUILD', CHECKOUTIFY_URL . '/build' );
		define( 'CHECKOUTIFY_ASSETS', CHECKOUTIFY_URL . '/assets' );
		define( 'CHECKOUTIFY_PRODUCTION', 'yes' );
		define( 'CHECKOUTIFY_AUTHOR_WPF', 'yes' );
		define( 'CHECKOUTIFY_POST_TYPE', 'rex_checkout' );
		define( 'CHECKOUTIFY_LAYOUT_PATH', CHECKOUTIFY_URL . '/includes/Assets/LayoutImages/' );
		define( 'CHECKOUTIFY_SETUP_WIZARD_IMAGES', CHECKOUTIFY_URL . '/includes/Assets/SetupWizardImages/' );

		// license constants

		$protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || !empty($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
		define( 'CHECKOUTIFY_INSTANCE', str_replace($protocol, "", get_bloginfo('wpurl')));
		define( 'CHECKOUTIFY_ITEM_ID', 'wpf-checkoutify' );
		define( 'CHECKOUTIFY_LICENSE_URL', 'https://useraccount.getwpfunnels.com/' );
		define( 'CHECKOUTIFY_LICENSE_MIDDLEMAN_URL', 'https://license.getwpfunnels.com/api/v1/licence' );

		define( 'REX_CHECKOUTIFY_ITEM_ID', '371622' );
		define( 'REX_CHECKOUTIFY_SL_STORE_URL', sanitize_url( 'https://rextheme.com/' ) );
	}

	/**
	 * Load the plugin after all plugins are loaded.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function init_plugin() {
		$this->includes();
		$this->init_hooks();
		$this->init_updater();

		/**
		 * Fires after the plugin is loaded.
		 *
		 * @since 1.0.0
		 */
		do_action( 'checkoutify_loaded' );
	}

	/**
	 * Activating the plugin.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function activate() {
		
	}

	/**
	 * Placeholder for deactivation function.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function deactivate() {
	}

	/**
	 * Flush rewrite rules after plugin is activated.
	 *
	 * Nothing being added here yet.
	 *
	 * @since 1.0.0
	 */
	public function flush_rewrite_rules() {
		// fix rewrite rules.
	}

	/**
	 * Run the installer to create necessary migrations and seeders.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	private function install() {
		$installer = new RexTheme\Checkoutify\Setup\Installer();
		$installer->run();
	}

	/**
	 * Include the required files.
	 *
	 * @since 0.2.0
	 *
	 * @return void
	 */
	public function includes() {
        if ( $this->is_request( 'admin' ) ) {
            $this->container['admin_menu'] = new RexTheme\Checkoutify\Admin\Menu();
			do_action('rex_checkoutify_loaded_admin',$this);
           
		}
	}

	/**
	 * Initialize the hooks.
	 *
	 * @since 0.2.0
	 *
	 * @return void
	 */
	public function init_hooks() {
		// Init classes.
		add_action( 'init', array( $this, 'init_classes' ) );

		// Add the plugin page links.
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );

        add_action('init', array( $this, 'load_checkoutify_textdomain') );

	}

    /**
     * Load the plugin text domain for translations.
     *
     * This method loads the text domain for the plugin, allowing for the translation of strings used within the plugin.
     * It specifies the location of the translation files within the 'languages' directory of the plugin.
     *
     * @since 1.0.6
     *
     * @return void
     */
    public function load_checkoutify_textdomain(){
        load_plugin_textdomain('checkoutify', false, dirname(plugin_basename(__FILE__)) . '/languages/');
    }

	/**
	 * Instantiate the required classes.
	 *
	 * @since 0.2.0
	 *
	 * @return void
	 */
	public function init_classes() {
		if ( $this->is_request( 'admin' ) ) {
			$this->container['admin_hook'] = new RexTheme\Checkoutify\Admin\Hooks();
		}

		$this->container['rest_api']     = new RexTheme\Checkoutify\REST\Api();
		$this->container['register_cpt'] = new RexTheme\Checkoutify\CPT\Register();
		$this->container['public_hook']   = new RexTheme\Checkoutify\Frontend\Hook();
		$this->container['compatibility'] = new RexTheme\Checkoutify\Compatibility\Compatibility();
	}

	/**
	 * What type of request is this.
	 *
	 * @since 0.2.0
	 *
	 * @param string $type admin, ajax, cron or frontend.
	 *
	 * @return bool
	 */
	private function is_request( $type ) {
		switch ( $type ) {
			case 'admin':
				return is_admin();

			case 'ajax':
				return defined( 'DOING_AJAX' );

			case 'rest':
				return defined( 'REST_REQUEST' );

			case 'cron':
				return defined( 'DOING_CRON' );

			case 'frontend':
				return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
		}
	}

	/**
	 * Modify the action links displayed for the plugin on the Plugins page.
	 *
	 * This method is responsible for adding custom action links to the plugin listing on the WordPress Plugins page.
	 * It allows users to easily access settings and documentation links.
	 *
	 * @param array $links An array of existing plugin action links.
	 *
	 * @return array An array of modified plugin action links, including custom links.
	 */
	public function plugin_action_links( $links ) {
		$links[] = '<a href="' . admin_url( 'admin.php?page=checkoutify#/license' ) . '">' . __( 'License', 'checkoutify' ) . '</a>';
		return $links;
	}


	/**
	 * Init updater class
	 *
	 * @since 1.0.0
	 */
	public function init_updater() {
		$license_key = trim( get_option( Keys::CHECKOUTIFY_LICENSE_KEY ) );
		if ( $license_key ) {
			new Updater(
				CHECKOUTIFY_LICENSE_URL,
				CHECKOUTIFY_FILE,
				array(
					'version' => CHECKOUTIFY_VERSION,
					'license' => $license_key,
					'item_id' => CHECKOUTIFY_ITEM_ID,
					'author'  => 'RexTheme',
					'url'     => home_url(),
					'beta'    => false,
				)
			);
		}
	}

    /**
     * Handle actions after the plugin is activated.
     *
     * This method checks if WooCommerce is installed and activated. If the plugin is activated and the license status is 'activate',
     * it redirects to the main plugin page. Otherwise, it redirects to the setup wizard.
     *
     * @param string $plugin The plugin being activated.
     * @since 1.0.3
     */
    public function rex_checkoutify_after_activation( $plugin ){
        if ( defined( 'WC_VERSION' ) && $plugin === plugin_basename( __FILE__ ) ) {
            if ( !empty( get_option('checkoutify_license_key') ) ) {
                wp_safe_redirect( esc_url( admin_url( 'admin.php?page=checkoutify#/' ) ) );
                exit;
            }
            wp_safe_redirect( esc_url( admin_url( 'admin.php?page=checkoutify#/setup-wizard' ) ) );
            exit;
        }
    }
}



/**
 * Initialize the main plugin.
 *
 * @since 1.0.0
 *
 * @return \Checkoutify|bool
 */
function checkoutify() {
	return Checkoutify::init();
}

/*
 * Kick-off the plugin.
 *
 * @since 1.0.0
 */
checkoutify();




















