<?php
/**
 * Assets
 *
 * @package RexTheme\Checkoutify\Assets
 * @version 1.0.0
 * @author A S M Nasim
 */

namespace RexTheme\Checkoutify\Assets;

use RexTheme\Checkoutify\Common\Functions;
use Exception;

	/**
	 * Vite class
	 *
	 * @package RexTheme\Checkoutify\Assets
	 * @version 1.0.0
	 * @author A S M Nasim
	 */
class Vite {

	/**
	 * The single instance of the class.
	 *
	 * @var Vite|null
	 */
	private static $instance = null;

	/**
	 * The protocol for the Vite host (e.g., 'http://' or 'https://').
	 *
	 * @var string
	 */
	private $viteHostProtocol = 'http://';

	/**
	 * The hostname or IP address of the Vite server.
	 *
	 * @var string
	 */
	private $viteHost = 'localhost';

	/**
	 * The port number on which the Vite server is running.
	 *
	 * @var string
	 */
	private $vitePort = '8880';

	/**
	 * The directory where Vite should find resources.
	 *
	 * @var string
	 */
	private $resourceDirectory = 'src/';

	/**
	 * An array of module scripts to be loaded.
	 *
	 * @var array
	 */
	private $moduleScripts = array();

	/**
	 * Indicates whether the script filter has been added.
	 *
	 * @var bool
	 */
	private $isScriptFilterAdded = false;


	/**
	 * Handle static method calls dynamically.
	 *
	 * This magic method is called when a static method is called on a class that cannot be found, and it allows dynamic
	 * handling of these method calls. In this implementation, it creates a single instance of the class (singleton pattern)
	 * and forwards the method call to that instance. If the class is not in development mode, it may also load a Vite
	 * manifest file.
	 *
	 * @param string $method The name of the method called.
	 * @param array  $params An array containing the method parameters.
	 *
	 * @return mixed The result of the method call on the class instance.
	 */
	public static function __callStatic( $method, $params ) {
		if ( null === static::$instance ) {
			static::$instance = new static();
			if ( !static::isDevMode() ) {
				( static::$instance )->viteManifest();
			}
		}
		return call_user_func_array( array( static::$instance, $method ), $params );
	}



	/**
	 * Enqueue a script.
	 *
	 * This method enqueues a script for both development and production modes. It checks if the script handle has already been used, and if so, it throws an exception in development mode.
	 *
	 * @param string $handle     Name for the script handle.
	 * @param string $src        URL of the script file.
	 * @param array  $dependency An array of registered script handles this script depends on.
	 * @param string $version    Script version number.
	 * @param bool   $inFooter   Whether to enqueue the script in the footer.
	 *
	 * @throws Exception If the script handle has been used in development mode.
	 *
	 * @return $this
	 */
	private function enqueueScript( $handle, $src, $dependency = array(), $version = null, $inFooter = false ) {
		if ( in_array( $handle, ( static::$instance )->moduleScripts, true ) ) {
			if ( static::isDevMode() ) {
				throw new Exception( 'This handel Has been used' );
			}
			return;
		}

		( static::$instance )->moduleScripts[] = $handle;

		if ( !( static::$instance )->isScriptFilterAdded ) {
			add_filter(
				'script_loader_tag',
				function ( $tag, $handle, $src ) {
					return ( static::$instance )->addModuleToScript( $tag, $handle, $src );
				},
				10,
				3
			);
			( static::$instance )->isScriptFilterAdded = true;
		}

		if ( !static::isDevMode() ) {
			$assetFile = ( static::$instance )->getFileFromManifest( $src );
			$srcPath   = static::getProductionFilePath( $assetFile, $version );
		} else {
			$srcPath = static::getDevPath() . $src;
		}

		wp_enqueue_script(
			$handle,
			$srcPath,
			$dependency,
			$version,
			$inFooter
		);

		$isRexthemeBuild = class_exists( '\RexTheme\Checkoutify\Rest\Actions\RexLicenseManager' ) && defined('CHECKOUTIFY_AUTHOR_REX');
		wp_localize_script(
			$handle,
			'adminLibrary',
			array(
				'ajaxUrl'              => esc_url_raw( admin_url( 'admin-ajax.php' ) ),
				'restApiUrl'           => esc_url_raw( get_rest_url() ),
				'homeUrl'              => esc_url_raw( home_url() ),
				'maybeWCActivate'      => Functions::maybeWCActivate(),
				'maybeLicenseActivate' => Functions::maybeLicenseActivate(),
				'maybeFunnelActivated' => Functions::maybeFunnelActivated(),
				'getPublishedFunnels'  => Functions::getPublishedFunnels(),
				'currency' 			   => get_woocommerce_currency_symbol(),
				'docUrl'               => $isRexthemeBuild ? esc_url( 'https://rextheme.com/docs-category/checkoutify/' ) : esc_url( 'https://getwpfunnels.com/docs/checkoutify-documentations-guides/' ),
				'supportUrl'           => $isRexthemeBuild ? esc_url( 'https://rextheme.com/support/' ) : esc_url( 'https://getwpfunnels.com/contact-us/' ),
				'manageLicenseUrl'     => $isRexthemeBuild ? esc_url( 'https://rextheme.com/your-account/' ) : esc_url( 'https://useraccount.getwpfunnels.com/orders/' ),
                'userInformation'      => Functions::get_logged_in_user_information(),
                'previewImageLink'     => CHECKOUTIFY_SETUP_WIZARD_IMAGES.'/setup-wizard-done-video-poster.webp',
                'woocommerceImageLink' => CHECKOUTIFY_SETUP_WIZARD_IMAGES.'/woocommerceLogo.webp',
                'setupWizardText'      => Functions::getSetupWizardText(),
                'checkoutifyLicenseKey'   => Functions::getCheckoutifyLicenseKey(),
                'adminUrl' => esc_url(get_admin_url()),
                'translationString' => Functions::checkoutifyTranslatableStrings(),
                'supportedFileTypes' => Functions::supportedFileTypes(),
                'isStripeActive'     => Functions::isStripeActive(),
			)
		);
		return $this;
	}

	/**
	 * Enqueue a stylesheet.
	 *
	 * This method enqueues a stylesheet for both development and production modes.
	 *
	 * @param string $handle     Name for the stylesheet handle.
	 * @param string $src        URL of the stylesheet file.
	 * @param array  $dependency An array of registered stylesheet handles this stylesheet depends on.
	 * @param string $version    Stylesheet version number.
	 *
	 * @return void
	 */
	private function enqueueStyle( $handle, $src, $dependency = array(), $version = null ) {
		if ( !static::isDevMode() ) {
			$assetFile = ( static::$instance )->getFileFromManifest( $src );
			$srcPath   = static::getProductionFilePath( $assetFile, $version );
		} else {
			$srcPath = static::getDevPath() . $src;
		}

		wp_enqueue_style(
			$handle,
			$srcPath,
			$dependency,
			$version
		);
	}

	/**
	 * Load Vite manifest data.
	 *
	 * This method loads and stores Vite manifest data, which maps entry points to their corresponding output files.
	 *
	 * @throws Exception If the Vite manifest file is not found.
	 *
	 * @return void
	 */
	private function viteManifest() {
		if ( !empty( ( static::$instance )->manifestData ) ) {
			return;
		}

		$manifestPath = realpath( __DIR__ ) . '/../../assets/manifest.json';
		if ( !file_exists( $manifestPath ) ) {
			throw new Exception( 'Vite Manifest Not Found. Run : npm run dev or npm run prod' );
		}
		$manifestFile                       = fopen( $manifestPath, 'r' ); //phpcs:ignore 
		$manifestData                       = fread( $manifestFile, filesize( $manifestPath ) ); //phpcs:ignore 
		( static::$instance )->manifestData = json_decode( $manifestData, true );
	}

	/**
	 * Get the asset file path from the Vite manifest.
	 *
	 * This method retrieves the asset file path from the Vite manifest based on the provided source URL. It checks if the asset exists in the manifest and throws an exception if not found when in dev mode.

	 * @param string $src The source URL of the asset.
	 * @return string The asset file path.
	 * @throws Exception If the asset file is not found in the Vite manifest.
	 */
	private function getFileFromManifest( $src ) {
		if ( !isset( ( static::$instance )->manifestData[ ( static::$instance )->resourceDirectory . $src ] ) && static::isDevMode() ) {
			throw new Exception( "$src file not found in vite manifest, Make sure it is in rollupOptions input and build again" );
		}

		return ( static::$instance )->manifestData[ ( static::$instance )->resourceDirectory . $src ];
	}

	/**
	 * Add 'type="module"' attribute to script tags for specific handles.
	 *
	 * This method adds the 'type="module"' attribute to script tags for specific script handles to indicate that they are ES6 modules.

	 * @param string $tag    The original script tag HTML.
	 * @param string $handle The script handle.
	 * @param string $src    The script source URL.

	 * @return string The modified script tag HTML.
	 */
	private function addModuleToScript( $tag, $handle, $src ) {
		if ( in_array( $handle, ( static::$instance )->moduleScripts, true ) ) {
			$tag = '<script type="module" src="' . esc_url( $src ) . '"></script>'; //phpcs:ignore 
		}
		return $tag;
	}

	/**
	 * Check if the plugin is in development mode.
	 *
	 * This method checks whether the plugin is in development mode by verifying if the 'CHECKOUTIFY_DEVELOPMENT' constant is defined
	 * and its value is set to 'yes'. Development mode is typically used for debugging and development purposes.
	 *
	 * @return bool True if the plugin is in development mode, false otherwise.
	 */
	public static function isDevMode(): bool {
		return defined( 'CHECKOUTIFY_DEVELOPMENT' ) && CHECKOUTIFY_DEVELOPMENT === 'yes';
	}

	/**
	 * Get the development path for Vite.
	 *
	 * This method constructs the development path for Vite by combining the Vite host protocol, host, port, and resource directory.
	 *
	 * @return string The development path for Vite.
	 */
	private static function getDevPath(): string {
		return ( static::$instance )->viteHostProtocol . ( static::$instance )->viteHost . ':' . ( static::$instance )->vitePort . '/' . ( static::$instance )->resourceDirectory;
	}

	/**
	 * Get the asset path.
	 *
	 * This method retrieves the asset path for Checkoutify, which is defined as the constant CHECKOUTIFY_ASSETS.
	 *
	 * @return string The asset path.
	 */
	private static function getAssetPath(): string {
		return CHECKOUTIFY_ASSETS;
	}

	/**
	 * Get the production file path for asset loading.
	 *
	 * This method is responsible for loading production assets and enqueuing CSS files.
	 *
	 * @param array  $file An array containing asset file details, including CSS files.
	 * @param string $version A string containing version.
	 * @return string The production file path for asset loading.
	 */
	private static function getProductionFilePath( $file, $version ): string {
		$assetPath = static::getAssetPath();
		if ( isset( $file['css'] ) && is_array( $file['css'] ) ) {
			foreach ( $file['css'] as $key => $path ) {
				wp_enqueue_style(
					$file['file'] . '_' . $key . '_css',
					$assetPath . '/' . $path,
					array(),
					$version
				); //phpcs:ignore 
			}
		}
		return ( $assetPath . '/' . $file['file'] );
	}
}
