<?php
/**
 * Manage common functions and utilities.
 *
 * This class contains various static methods for handling common functionality and generating response arrays.
 *
 * @package RexTheme\Checkoutify\Common
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Common;

use Error;

use function Patchwork\Stack\push;
use RexTheme\Checkoutify\Common\Keys;
use WP_Query;

/**
 * A utility class for common functions and response generation.
 *
 * This class contains various static methods for handling common functionality and generating response arrays.
 *
 * @since 1.0.0
 * @package RexTheme\Checkoutify\Common
 */
class Functions {


	/**
	 * Get a success response array.
	 *
	 * This method generates a success response array with a 'success' key set to true.
	 *
	 * @param array $property (Optional) Additional properties to include in the response array.
	 * @return array An array representing a successful response.
	 *
	 * @since 1.0.0
	 */
	public static function getSuccessResponse( $property = array() ) {
		$response = array(
			'success' => true,
		);
		return array_merge( $response, $property );
	}


	/**
	 * Get an error response array.
	 *
	 * This method generates an error response array with an 'error' key set to false.
	 *
	 * @param array $property (Optional) Additional properties to include in the response array.
	 * @return array An array representing an error response.
	 *
	 * @since 1.0.0
	 */
	public static function getErrorResponse( $property = array() ) {
		$response = array(
			'error' => true,
		);
		return array_merge( $response, $property );
	}

	/**
	 * Check if WooCommerce is activated.
	 *
	 * This function checks if the WooCommerce plugin is activated on the WordPress site.
	 *
	 * @return bool True if WooCommerce is activated, false otherwise.
	 *
	 * @since 1.0.0
	 */
	public static function maybeWCActivate() {
		if ( !defined( 'WC_PLUGIN_FILE' ) ) {
			return false;
		}
		return true;
	}

	/**
	 * Get metadata associated with a checkout.
	 *
	 * Retrieves metadata associated with a specific checkout by its ID and optional meta key.
	 *
	 * @param int    $id      The ID of the checkout.
	 * @param string $metaKey (Optional) The specific meta key to retrieve. If not provided, all metadata is retrieved.
	 *
	 * @return array An array containing the requested metadata or with an error message.
	 *
	 * @since 1.0.0
	 */
	public static function getMetaData( $id, $metaKey = '' ) {
		if ( !$id ) {
			$property = array(
				'type' => 'No checkout ID is found',
			);
			return self::getErrorResponse( $property );
		}
		$metaData = get_post_meta( $id, $metaKey, true );
		if ( is_wp_error( $metaData ) ) {
			$property = array(
				'type' => 'Incorrect checkout ID',
			);
			return self::getErrorResponse( $property );
		}

		if ( !$metaKey && is_array( $metaData ) ) {
			foreach ( $metaData as $key => $data ) {
				if ( isset( $data[0] ) ) {
					$metaData[ $key ] = maybe_unserialize( $data[0] );
				}
			}
		}
		$property = array(
			'metas' => $metaData,
		);
		return self::getSuccessResponse( $property );
	}


	/**
	 * Generates CSS styles based on the provided style attributes and input type.
	 *
	 * This function takes an associative array of style attributes and their values
	 * and generates CSS styles. Depending on the input type specified, it can generate
	 * styles for regular elements or styles specific to checkbox and radio input types.
	 *
	 * @param array  $styles An associative array of style attributes and their values.
	 * @param string $inputType The type of input element ('checkbox', 'radio', or empty for regular elements).
	 *
	 * @return array An array containing the generated CSS styles. For regular elements,
	 *               the array contains a 'style' key with the styles. For checkbox and
	 *               radio input types, the array contains 'styleTitle' for label styles,
	 *               'styleBox' for the outer box styles, and 'styleInner' for the inner box styles.
	 */
	public static function getStyles( $styles, $inputType = '' ) {
		if ( empty( $styles ) ) {
			return array();
		}

		$inputLabelTop     = array( 'top' );
		$inputLabelBg      = array( 'backgroundColor' );
		$inputLabelProperty= array( 'fontSize', 'lineHeight', 'textTransform', 'textDecoration', 'fontWeight', 'color', 'marginBottom' );

		$titleFor          = array( 'fontSize', 'fontFamily', 'fontWeight', 'color', 'textTransform' );
		$innerFor          = array( 'borderRadius', 'activeIconColor', 'activeIconHeight', 'activeIconWidth' );
		$select2Style      = array( 'borderRadius', 'borderStyle', 'borderWidth', 'borderColor', 'backgroundColor' );
		$select2StyleInner = array( 'fontSize', 'lineHeight', 'color', 'paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft' );
		$boxFor            = array( 'backgroundColor', 'borderRadius', 'borderColor', 'styleBox', 'borderStyle', 'borderWidth', 'height', 'width', 'activeBgColor', 'activeBorderColor' );

		$response          = array();
		if ( 'checkbox' === $inputType || 'radio' === $inputType ) {
			$response = array(
				'styleTitle' => '',
				'styleInner' => '',
				'styleBox'   => '',
			);
		} elseif ( 'select2' === $inputType ) {
			$response = array(
				'select2StyleInner' => '',
				'select2Style'      => '',
			);
		} elseif( 'titleTop' === $inputType ){
			$response = array(
				'inputLabelTop' => '',
			);
			
		} elseif( 'titleBgColor' === $inputType ){
			$response = array(
				'inputLabelBgColor' => '',
			);

		}elseif( 'inputLabelProperty' === $inputType ){
			$response = array(
				'inputLabelProperty' => '',
			);

		} else {
			$response = array(
				'style' => '',
			);
		}
		if ( !empty( $styles ) && is_array( $styles ) ) {
			$formatedStyle = self::getFormatedStyle();
			foreach ( $styles  as $styleKey => $styleValue ) {
				if ( isset( $formatedStyle[ $styleKey ] ) ) {
					$value = is_array( $styleValue ) && isset( $styleValue ['value'] ) ? $styleValue ['value'] : $styleValue;

					if ( 'checkbox' === $inputType || 'radio' === $inputType ) {
						if ( in_array( $styleKey, $titleFor ) ) {
							// $response['styleTitle'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['styleInner'] .= isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] . ':' : ' ';
							$response['styleInner'] .= isset($value) ? $value : '';
							$response['styleInner'] .= isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ';
							$response['styleInner'] .= isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : '';
							$response['styleInner'] .= '; ';

						}
						if ( in_array( $styleKey, $innerFor ) ) {
							// $response['styleInner'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['styleInner'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                         							(isset($value) ? $value : '') .
                         							(isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                         							(isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : ' ') . '; ';

						}
						if ( in_array( $styleKey, $boxFor ) ) {
							// $response['styleBox'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['styleBox'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                        (isset($value) ? $value : '') .
                        (isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                        (isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : ' ') . '; ';

						}
					} elseif ( 'select2' === $inputType ) {
						if ( in_array( $styleKey, $select2StyleInner ) ) {
							// $response['select2StyleInner'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['select2StyleInner'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                                  (isset($value) ? $value : '') .
                                  (isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                                  (isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : ' ') . '; ';

						}
						if ( in_array( $styleKey, $select2Style ) ) {
							// $response['select2Style'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['select2Style'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                             (isset($value) ? $value : '') .
                             (isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                             (isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : ' ') . '; ';

						}

					} elseif ( 'titleTop' === $inputType ){
						if ( in_array( $styleKey, $inputLabelTop ) ) {
							$response['inputLabelTop'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . ' ' . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
						}

					} elseif ( 'titleBgColor' === $inputType ){
						if ( in_array( $styleKey, $inputLabelBg ) ) {
							// $response['inputLabelBgColor'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . ' ' . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['inputLabelBgColor'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                                   (isset($value) ? $value : '') .
                                   (isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                                   ' ' . (isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : '') . '; ';

						}

					} elseif ( 'inputLabelProperty' === $inputType ){
						if ( in_array( $styleKey, $inputLabelProperty ) ) {
							// $response['inputLabelProperty'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . ' ' . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
							$response['inputLabelProperty'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                                   (isset($value) ? $value : '') .
                                   (isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                                   ' ' . (isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : '') . '; ';

						}

					} else {
						// $response['style'] .= $formatedStyle[ $styleKey ]['property'] . ':' . $value . $formatedStyle[ $styleKey ]['unit'] . ' ' . $formatedStyle[ $styleKey ]['isImportant'] . '; ';
						$response['style'] .= (isset($formatedStyle[$styleKey]['property']) ? $formatedStyle[$styleKey]['property'] : ' ') . ':' .
                      (isset($value) ? $value : '') .
                      (isset($formatedStyle[$styleKey]['unit']) ? $formatedStyle[$styleKey]['unit'] : ' ') .
                      ' ' . (isset($formatedStyle[$styleKey]['isImportant']) ? $formatedStyle[$styleKey]['isImportant'] : '') . '; ';

					}
				}
			}
			return $response;
		}
	}

	/**
	 * Get formatted style properties.
	 *
	 * @return array An associative array containing various style properties with default values.
	 */
	public static function getFormatedStyle() {
		$style = array(
			'fontSize'          => array(
				'value'       => '',
				'property'    => 'font-size',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'fontWeight'        => array(
				'value'       => '',
				'property'    => 'font-weight',
				'unit'        => '',
				'isImportant' => '!important',
			),
			'color'             => array(
				'value'       => '',
				'property'    => 'color',
				'unit'        => '',
				'isImportant' => '!important',
			),
			'backgroundColor'   => array(
				'value'       => '',
				'property'    => 'background-color',
				'unit'        => '',
				'isImportant' => '',
			),
			'borderRadius'      => array(
				'value'       => '',
				'property'    => 'border-radius',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'borderColor'       => array(
				'value'       => '',
				'property'    => 'border-color',
				'unit'        => '',
				'isImportant' => '',
			),
			'borderWidth'       => array(
				'value'       => '',
				'property'    => 'border-width',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'paddingTop'        => array(
				'value'       => '',
				'property'    => 'padding-top',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'paddingRight'      => array(
				'value'       => '',
				'property'    => 'padding-right',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'paddingBottom'     => array(
				'value'       => '',
				'property'    => 'padding-bottom',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'paddingLeft'       => array(
				'value'       => '',
				'property'    => 'padding-left',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'letterSpacing'     => array(
				'value'       => '',
				'property'    => 'letter-spacing',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'borderStyle'       => array(
				'value'       => '',
				'property'    => 'border-style',
				'unit'        => '',
				'isImportant' => '!important',
			),
			'gap'               => array(
				'value'       => '',
				'property'    => 'gap',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'height'            => array(
				'value'       => '',
				'property'    => 'height',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'width'             => array(
				'value'       => '',
				'property'    => 'width',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'textTransform'     => array(
				'value'       => '',
				'property'    => 'text-transform',
				'unit'        => '',
				'isImportant' => '!important',
			),
			'textDecoration'    => array(
				'value'       => '',
				'property'    => 'text-decoration',
				'unit'        => '',
				'isImportant' => '!important',
			),
			'lineHeight'        => array(
				'value'       => '',
				'property'    => 'line-height',
				'unit'        => '',
				'isImportant' => '!important',
			),
			'marginBottom'      => array(
				'value'       => '',
				'property'    => 'margin-bottom',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'marginLeft'        => array(
				'value'       => '',
				'property'    => 'margin-left',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'activeBgColor'     => array(
				'value'       => '',
				'property'    => '--activeBgColor',
				'unit'        => '',
				'isImportant' => '',
			),
			'activeBorderColor' => array(
				'value'       => '',
				'property'    => '--activeBorderColor',
				'unit'        => '',
				'isImportant' => '',
			),
			'activeIconColor'   => array(
				'value'       => '',
				'property'    => 'background-color',
				'unit'        => '',
				'isImportant' => '',
			),
			'activeIconHeight'  => array(
				'value'       => '',
				'property'    => 'height',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'activeIconWidth'   => array(
				'value'       => '',
				'property'    => 'width',
				'unit'        => 'px',
				'isImportant' => '!important',
			),
			'top'               => array(
				'value'       => '',
				'property'    => 'top',
				'unit'        => 'px',
				'isImportant' => '',
			),
		);

		return apply_filters( 'checkoutify_fields_style_property', $style );
	}

	/**
	 * Generates CSS rules for a pseudo-class selector based on the provided CSS properties.
	 *
	 * This function takes an array of CSS properties and generates CSS rules for a pseudo-class selector.
	 * The selector is constructed based on the input type and selector name.
	 *
	 * @param array  $cssProperties An array of CSS properties to apply to the pseudo-class selector.
	 * @param string $selectorName The name of the selector to which the pseudo-class rules will be applied.
	 * @param string $inputType The type of input element for which the pseudo-class rules are generated.
	 *
	 * @return string The generated CSS rules wrapped in a style tag, or an empty string if no properties are provided.
	 */
	public static function getPseudoStyle( $cssProperties, $selectorName, $inputType ) {
		$selector = 'input[type="' . $inputType . '"]:checked' . ' + .' . $selectorName;
		$cssRules = '';
		if ( !empty( $cssProperties ) && is_array( $cssProperties ) ) {
			$generatedCss = '';
			foreach ( $cssProperties as $property ) {
				$generatedCss .= ' ' . $property . ' ';
			}
			$cssRules .= $selector . ' { ' . $generatedCss . ' } ';
			return '<style> ' . $cssRules . '</style>';
		}
		return $cssRules;
	}


	/**
	 * Get the checkoutify_id from the $_POST data.
	 *
	 * This method extracts and sanitizes the 'post_data' parameter from the $_POST array,
	 * then parses it to retrieve the 'checkoutify_id'. If the 'checkoutify_id' is found,
	 * it is returned; otherwise, false is returned.
	 *
	 * @since 1.0.0
	 *
	 * @return mixed|string|false The checkoutify_id if found, otherwise false.
	 */
	public static function getCheckoutifyIdFromPost() {
		
		if ( isset( $_POST['post_data'] ) ) {
			$post_data = array();
			parse_str( sanitize_text_field( $_POST['post_data'] ), $post_data );
			
			if ( isset( $post_data['checkoutify_id'] ) ) {
				
				return $post_data['checkoutify_id'];
			}
		}
		return false;
	}


	/**
	 * Check if Checkoutify Shortcode is Available.
	 *
	 * This function checks if the 'checkoutify' shortcode is present in the post content.
	 *
	 * @return bool Whether the 'checkoutify' shortcode is available in the post content.
	 *
	 * @since 1.0.0
	 */
	public static function isCheckoutifyShortcodeAvailable() {
		// Use the has_shortcode function to check if the 'checkoutify' shortcode is present in the post content.
		if ( has_shortcode( get_the_content(), 'checkoutify' ) ) {
			return true;
		} else {
			if ( self::getCheckoutifyIdFromPost() ) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Check the license is activate or not
	 *
	 * @return bool If license status is valid then return true otherwise false
	 * @since 1.0.0
	 */
	public static function maybeLicenseActivate() {
		return 'activate' === get_option( Keys::CHECKOUTIFY_LICENSE_STATUS, 'deactivate' );
	}


	/**
	 * Check the elemntor plugin is activate or not
	 *
	 * @return bool If activate then return true otherwise false
	 * @since 1.0.0
	 */
	public static function maybeElementorActivated() {
		return defined( 'ELEMENTOR_VERSION' );
	}
	
	
	/**
	 * Check the bricks is activate or not
	 * 
	 * @return bool If activate then return true otherwise false
	 * @since 1.0.0 
	 */
	public static function maybeBricksActivated(){
		return 'bricks' === wp_get_theme()->template;
	}


	/**
	 * Relocate WooCommerce template files.
	 *
	 * This function checks if a custom template exists in the Checkoutify plugin directory and uses it instead of the default WooCommerce template.
	 *
	 * @param string $template The template file.
	 * @param string $templateName The name of the template.
	 * @param string $templatePath The path to the template.
	 * @return string The relocated template path.
	 *
	 * @since 1.0.0
	 */
	public static function relocateWoocommerceTemplate( $template, $templateName, $templatePath ) {
		// Store the original template path.
		$_template  = $template;
		$pluginPath = CHECKOUTIFY_DIR . '/woocommerce/templates/';
		// Check if a custom template exists in the plugin directory.
		if ( file_exists( $pluginPath . $templateName ) ) {
			$template = $pluginPath . $templateName;
		}

		// If the template is not found, revert to the original path.
		if ( !$template ) {
			$template = $_template;
		}

		return $template;
	}


	/**
	 * Get the default checkout ID if available.
	 *
	 * This function checks conditions related to the WPFunnels plugin and the status of the default checkout.
	 *
	 * @return int|bool The checkout ID if available, or false if not.
	 * @since 1.0.0
	 */
	public static function getDefaultCheckout(){
		$defaultCheckout = get_option( Keys::CHECKOUTIFY_DEFAULT_CHECKOUT_KEY );
		if ( !defined( 'WPFNL_VERSION' ) || ( defined( 'WPFNL_VERSION' ) && !\WPFunnels\Wpfnl_functions::check_if_this_is_step_type( 'checkout' )  )  ){
			if( $defaultCheckout && 'publish' === get_post_status( $defaultCheckout ) ){
				return $defaultCheckout;
			}
		}
		return false;
	}

	/**
    * Check if WP Funnels plugin is activated.
    *
    * @return bool Returns true if the WP Funnels plugin is active, otherwise false.
    */
	public static function maybeFunnelActivated() {
		if ( !defined( 'WPFNL_FILE' ) ) {
			return false;
		}
		return true;
	}
 
 
	/**
	 * Get a list of published funnels.
	 *
	 * @return array An associative array of funnel IDs and titles.
	 */
	public static function getPublishedFunnels() {
		$funnelList = array(
			"#" => "Select a funnel"
		);

		// Check if the WP Funnels plugin is activated
		if (self::maybeFunnelActivated()) {
			$args = array(
				'post_type'      => 'wpfunnels',
				'post_status'    => 'publish',
				'posts_per_page' => -1,
			);

			// Query for published funnels
			$query = new WP_Query($args);

			while ($query->have_posts()) {
				$query->the_post();
				$post_id    = get_the_ID();
				$post_title = get_the_title();

				// Add funnel ID and title to the array
				$funnelList[$post_id] = $post_title;
			}

		}

		return $funnelList;
	}

	/**
	 * Check if a checkout has an associated funnel ID in the WordPress post meta.
	 *
	 * This function checks if a checkout with the given ID has an associated funnel ID in the
	 * WordPress post meta. It returns the associated funnel ID if found, or false if no association
	 * is present. The function ensures that the checkout ID is provided before attempting to fetch
	 * the associated funnel ID.
	 *
	 * @param int $checkoutId The ID of the checkout to check for an associated funnel.
	 *
	 * @return mixed           The associated funnel ID if found, or false if not found.
	 *
	 * @since 1.0.0
	 */
	public static function maybeCheckoutAvailableByFunnelId( $funnelId ){

		if( !$funnelId ){

			return false;

		}

		$posts = get_posts([
			'post_type' => 'rex_checkout',
			'post_status' => 'publish',
			'orderby' => 'modified',
			'order' => 'DESC',
			'posts_per_page' => -1
		  ]);

		$checkoutId = false;
		
		if( !empty( $posts ) ){
			foreach( $posts as $post){
				$id = isset( $post->ID ) ? $post->ID : '';
				$metaValue = get_post_meta($id, Keys::CHECKOUTIFY_FUNNEL_ID, true);
				if( isset( $metaValue ) && $metaValue === $funnelId ){
					$checkoutId = $id;
					break;
				}
			}
		}
		
		return $checkoutId;

	}

    /**
     * Retrieve information about the currently logged-in user.
     *
     * This method fetches the current WordPress user and returns an array containing
     * the user's email and display name.
     *
     * @since 1.0.3
     * @return array An array containing the user's email and display name.
     */
    public static function get_logged_in_user_information(): array
    {
        $admin_user = wp_get_current_user();
        return array(
            'email' => !empty( $admin_user->user_email ) ? $admin_user->user_email : '',
            'name' => !empty( $admin_user->display_name ) ? $admin_user->display_name : '',
        );
    }

    public static function getSetupWizardText(){
        $data = [
            'doneText' => __("Done", 'checkoutify'),
            'pluginsAndLicenseText' => __("Plugins & License", 'checkoutify'),
            'welcomeText' => __("Welcome", 'checkoutify'),
            "stepOne" => [
                "welcome" => __("Hello, welcome to ", 'checkoutify'),
                "checkoutify" => __("Checkoutify!", 'checkoutify'),
                "description" => __("Checkoutify is a game-changing WooCommerce checkout customizer that lets you to create and replace your default checkout process with a well organized, drag-and-drop experience.", 'checkoutify'),
                "newButtonText" => __("Let's customize your checkout", 'checkoutify'),
                "guideButtonText" => __("Check the guide", 'checkoutify'),
                "featuresHeading" => __('Features', 'checkoutify'),
                "featuresSubHeading" => __('Checkoutify - The Ultimate WooCommerce Checkout Customizer', 'checkoutify'),
                "featureOneTitle" => __('Drag and Drop Editor', 'checkoutify'),
                "featureOneDescription" => __('Drag and Drop Editor for the Checkout Editor.', 'checkoutify'),
                "featureTwoTitle" => __('Multi-Step Checkout', 'checkoutify'),
                "featureTwoDescription" => __("Provide a smooth and organized checkout process with Checkoutify's multi-step checkout feature.", 'checkoutify'),
                "featureThreeTitle" => __('Remove Unnecessary Checkout Fields', 'checkoutify'),
                "featureThreeDescription" => __("Remove unnecessary checkout fields for a clutter-free experience.", 'checkoutify'),
                "featureFourTitle" => __('Guest Checkout Friendly', 'checkoutify'),
                "featureFourDescription" => __("Serve both registered and guest customers with Checkoutify's guest checkout-friendly interface.", 'checkoutify'),
                "featureFiveTitle" => __('Multiple Payment Gateway', 'checkoutify'),
                "featureFiveDescription" => __("Integrate Checkoutify with a wide range of popular payment gateways.", 'checkoutify'),
                "featureSixTitle" => __('Rearrange Steps', 'checkoutify'),
                "featureSixDescription" => __("Rearrange Steps, Move Sections: Customize the Journey.", 'checkoutify'),
                "featureSevenTitle" => __('Default WooCommerce Checkout', 'checkoutify'),
                "featureSevenDescription" => __("With just one simple click, set your customized checkout experience as the default checkout process for your WooCommerce store.", 'checkoutify'),
                "featureEightTitle" => __('Independent Order Bump', 'checkoutify'),
                "featureEightDescription" => __("Independent Order Bump: Boost Sales Strategically.", 'checkoutify'),
                "featureButtonText" => __("Check All Features", 'checkoutify'),
                "nextButtonText" => __("Next", 'checkoutify'),
            ],
            "stepTwo" =>[
                "necessary" => __("Necessary", 'checkoutify'),
                "plugin" => __("Plugin", 'checkoutify'),
                "wooCommerce" => __("WooCommerce", 'checkoutify'),
                "requiredText" => __("Required for Checkoutify.", 'checkoutify'),
                "installed" => __("INSTALLED", 'checkoutify'),
                "required" => __("REQUIRED", 'checkoutify'),
                "activateText" => __("Activate your", 'checkoutify'),
                "license" => __("License", 'checkoutify'),
                "activateButtonText" => __("Activate License", 'checkoutify'),
                "deactivateButtonText" => __("Deactivate License", 'checkoutify'),
                "licenseInputPlaceholder" => __("Enter your license key", 'checkoutify'),
                "invalidLicenseMessage" => __("Invalid license key", 'checkoutify'),
                "validLicenseMessage" => __("Your license is activated", 'checkoutify'),
                'letsCustomizeYourCheckout' => __("Let's customize your checkout", 'checkoutify'),
            ],
            "stepThree" =>[
                "youAreText" => __("You Are ", 'checkoutify'),
                "doneText" => __("Done", 'checkoutify'),
                "testimonials" => __("Testimonials", 'checkoutify'),
                "reviewerOneText" => __("Working fine & nice support.", 'checkoutify'),
                "reviewerOneName" => __("- Francesco Marcuzzi", 'checkoutify'),
                "reviewerTwoText" => __("Good plugin, good service.", 'checkoutify'),
                "reviewerTwoName" => __("- Aleks1180", 'checkoutify'),
                "optinButtonText" => __("Opt-in to receive tips, discounts, and recommendations from the RexTheme team directly in your inbox.", 'checkoutify'),
            ]
        ];

        return $data;
    }

    /**
     * Retrieve the Checkoutify license key.
     *
     * This method fetches the Checkoutify license key from the WordPress options table.
     *
     * @since 1.0.3
     * @return string The Checkoutify license key if it exists, otherwise an empty string.
     */
    public static function getCheckoutifyLicenseKey(){
        return get_option( Keys::CHECKOUTIFY_LICENSE_KEY, '' );
    }

    /**
     * Returns an array of translatable strings used in the Checkoutify plugin.
     *
     * This function provides a centralized location for all the translatable strings
     * used throughout the Checkoutify plugin. Each string is associated with a specific
     * context or section of the plugin, making it easier to manage translations.
     *
     * @since 1.0.6
     *
     * @return array An associative array of translatable strings.
     */
    public static function checkoutifyTranslatableStrings(){
        $translatableStrings = [
            'dashboard' => [
                'checkouts' => __('Checkouts', 'checkoutify'),
                'add_new' => __('Add New', 'checkoutify'),
                'list_view' => __('List View', 'checkoutify'),
                'search_form' => __('Search form', 'checkoutify'),
                'filter_options' => [
                    'all' => __('All', 'checkoutify'),
                    'published' => __('Published', 'checkoutify'),
                    'draft' => __('Draft', 'checkoutify'),
                    'trash' => __('Trash', 'checkoutify'),
                ],
                'name' => __('Name', 'checkoutify'),
                'last_update' => __('Last Update', 'checkoutify'),
                'status' => __('Status', 'checkoutify'),
                'shortcode' => __('Shortcode', 'checkoutify'),
                'action' => __('Action', 'checkoutify'),
                'delete_selected' => __('Delete Selected', 'checkoutify'),
                'noCheckoutPageAvailableText' => __('No Checkout page available', 'checkoutify'),
            ],
            'checkoutSingle' => [
                'createCheckoutText' => __('Create Checkout', 'checkoutify'),
                'duplicateButtonText' => __('Duplicate', 'checkoutify'),
                'duplicatingButtonText' => __('Duplicating...', 'checkoutify'),
            ],
            'CheckoutCreateModal'=> [
                'nameToYourCheckout' => __('Name to your checkout', 'checkoutify'),
                "createCheckoutText" => __("Create Checkout", 'checkoutify'),
                "creatingCheckout" => __("Creating...", 'checkoutify'),
                "enterCheckoutName" => __("Enter checkout name", 'checkoutify'),
            ],
            'checkoutDeleteModal' => [
                'deleteCheckoutText' => __('Delete Checkout', 'checkoutify'),
                'deleteCheckoutMessage' => __('Are you sure you want to delete this checkout?', 'checkoutify'),
                'deleteCheckoutsMessage' => __('Are you sure you want to delete these checkouts?', 'checkoutify'),
                'deletingCheckout' => __('Deleting...', 'checkoutify'),
                'deleteBtnText' => __('Delete', 'checkoutify'),
                'cancelBtnText' => __('Cancel', 'checkoutify'),
                'deleteCheckoutsText' => __('Delete Checkouts', 'checkoutify'),
            ],
            'navigation' => [
                'home' => __('Home', 'checkoutify'),
                'license' => __('License', 'checkoutify'),
                'setupWizard' => __('Setup Wizard', 'checkoutify'),
            ],
            'checkoutLayouts' => [
                'findYourLayout' => __('Find Your Layout', 'checkoutify'),
                'startFromScratch' => __('Start From Scratch', 'checkoutify'),
                'title' => __('Title', 'checkoutify'),
            ],
            'checkoutLayoutSingle' => [
                'import' => __('Import', 'checkoutify'),
                'importing' => __('Importing', 'checkoutify'),
            ],
            'styleSettings' => [
                'fontSizeText' => __('Font size (px)', 'checkoutify'),
                'fontWeightText' => __('Font weight', 'checkoutify'),
                'fontColorText' => __('Font color', 'checkoutify'),
                'textTransformText' => __('Text Transform', 'checkoutify'),
                'lineHeightText' => __('Line height', 'checkoutify'),
                'borderRadiusText' => __('Border Radius (px)', 'checkoutify'),
                'backgroundColorText' => __('Background Color', 'checkoutify'),
                'paddingText' => __('Padding', 'checkoutify'),
                'paddingTopText' => __('Top (px)', 'checkoutify'),
                'paddingRightText' => __('Right (px)', 'checkoutify'),
                'paddingBottomText' => __('Bottom (px)', 'checkoutify'),
                'paddingLeftText' => __('Left (px)', 'checkoutify'),
                'fontWeight400' => __('400', 'checkoutify'),
                'fontWeight500' => __('500', 'checkoutify'),
                'fontWeight600' => __('600', 'checkoutify'),
                'fontWeight700' => __('700', 'checkoutify'),
                'fontWeight800' => __('800', 'checkoutify'),
                'fontWeight900' => __('900', 'checkoutify'),
                'textDecorationDefault' => __('Default', 'checkoutify'),
                'textDecorationUnderline' => __('Underline', 'checkoutify'),
                'textDecorationOverline' => __('Overline', 'checkoutify'),
                'textDecorationLineThrough' => __('Line Through', 'checkoutify'),
                'textTransformDefault' => __('Default', 'checkoutify'),
                'textTransformUppercase' => __('Uppercase', 'checkoutify'),
                'textTransformLowercase' => __('Lowercase', 'checkoutify'),
                'textTransformCapitalize' => __('Capitalize', 'checkoutify'),
                'borderStyleNone' => __('Default', 'checkoutify'),
                'borderStyleSolid' => __('Solid', 'checkoutify'),
                'borderStyleDotted' => __('Dotted', 'checkoutify'),
                'borderStyleDashed' => __('Dashed', 'checkoutify'),
                'borderText' => __('Border', 'checkoutify'),
                'styleText' => __('Style', 'checkoutify'),
                'colorText' => __('Color', 'checkoutify'),
                'widthText' => __('Width (px)', 'checkoutify'),
                'textDecorationText' => __('Text Decoration', 'checkoutify'),
                'labelSpacingText' => __('Label Spacing', 'checkoutify'),
                'buttonText' => __('Button Text', 'checkoutify'),
                'checkIconColorText' => __('Check Icon Color', 'checkoutify'),
                'borderColorText' => __('Border Color', 'checkoutify'),
                'checkIconHeightText' => __('Check Icon Height (px)', 'checkoutify'),
                'checkIconWidthText' => __('Check Icon Width (px)', 'checkoutify'),
                'heightText' => __('Height (px)', 'checkoutify'),
                'checkboxActiveStyleText' => __('Checkbox Active style', 'checkoutify'),
                'arrowColorText' => __('Arrow Color', 'checkoutify'),
                'topDistanceText' => __('Top Distance (px)', 'checkoutify'),
                'positionText' => __('Position', 'checkoutify'),
                'outsideText' => __('Outside', 'checkoutify'),
                'insideText' => __('Inside', 'checkoutify'),
                'floatingText' => __('Floating', 'checkoutify'),
                'marginBottomText' => __('Margin Bottom', 'checkoutify'),
                'iconWidthText' => __('Icon Width (px)', 'checkoutify'),
                'iconHeightText' => __('Icon Height (px)', 'checkoutify'),
                'iconColorText' => __('Icon Color', 'checkoutify'),
                'radioActiveStyleText' => __('Radio Active style', 'checkoutify'),
                'gapText' => __('Gap (px)', 'checkoutify'),
                'layoutText' => __('Layout', 'checkoutify'),
                'horizontalText' => __('Horizontal', 'checkoutify'),
                'verticalText' => __('Vertical', 'checkoutify'),
                'letterSpacing' => __('Letter Spacing', 'checkoutify'),
                'radioButtonActiveStyleText' => __('Radio Button Active style', 'checkoutify'),
                'inputStyleText' => __('Input style', 'checkoutify'),
                'labelStyleText' => __('Label style', 'checkoutify'),
                'arrowSizeText' => __('Arrow Size', 'checkoutify'),
                'stepDesignText' => __('Step Design', 'checkoutify'),
                'activeOrCompletedColor' => __('Active/Completed Color', 'checkoutify'),
            ],
            'fieldSettingDrawer' => [
                'labelText' => __('Label', 'checkoutify'),
                'inputText' => __('Input', 'checkoutify'),
                'checkboxText' => __('Checkbox', 'checkoutify'),
                'placeOrderButtonText' => __('Place Order Button', 'checkoutify'),
                'applyToAllText' => __('Apply to all', 'checkoutify'),
                'productText' => __('Product', 'checkoutify'),
                'subTotalText' => __('Subtotal', 'checkoutify'),
                'shippingMethodText' => __('Shipping Method', 'checkoutify'),
                'totalText' => __('Total', 'checkoutify'),
                'buttonText' => __('Button', 'checkoutify'),
                'radioButtonText' => __('Radio Button', 'checkoutify'),
            ],
            'orderProductSettings' => [
                'productImageBorderColorText' => __('Product image border color', 'checkoutify'),
                'enableOrDisableProduct' => __('Enable/Disable Product image:', 'checkoutify'),
            ],
            'multiStepNavigation' =>[
                'nextPreviousButtonDesign' => __('Next/Previous Button Design', 'checkoutify'),
                'previousButtonText' => __('Previous button text', 'checkoutify'),
                'nextButtonText' => __('Next button text', 'checkoutify'),
            ],
            'orderBumpCreateModal' => [
                'addOrderBumpText' => __('Add Order Bump', 'checkoutify'),
                'selectTemplateText' => __('Select Template', 'checkoutify'),
                'orderBumpBackgroundColorText' => __('Order Bump Background Color', 'checkoutify'),
                'orderBumpPrimaryColorText' => __('Order Bump Primary Color', 'checkoutify'),
                'orderBumpPriceColorText' => __('Order Bump Price Color', 'checkoutify'),
                'selectProductTextText' => __('Select Product', 'checkoutify'),
                'productTitleText' => __('Product Title', 'checkoutify'),
                'productQuantityText' => __('Product Quantity', 'checkoutify'),
                'productImageText' => __('Product Image', 'checkoutify'),
                'browseAnImageText' => __('Browse an image', 'checkoutify'),
                'updateTheImageText' => __('Update the Image', 'checkoutify'),
                'recommendedImageText' => __('Recommended image size 150px x 150px.', 'checkoutify'),
                'highlightText' => __('Highlight Text', 'checkoutify'),
                'checkboxLabelText' => __('Checkbox Label', 'checkoutify'),
                'productDescriptionText' => __('Product Description', 'checkoutify'),
                'discountTypeText' => __('Discount Type', 'checkoutify'),
                'discountApplyToText' => __('Discount Apply To', 'checkoutify'),
                'discountValueText' => __('Discount Value', 'checkoutify'),
                'originalPriceText' => __('Original Price', 'checkoutify'),
                'salePriceText' => __('Sale Price', 'checkoutify'),
                'discountedOfferPriceText' => __('Discounted Offer Price', 'checkoutify'),
                'replaceProductText' => __('Replace Product', 'checkoutify'),
                'warningMessageText' => __('It will replace all products (from checkout products) with the order bump product.', 'checkoutify'),
                'originalText' => __('Original', 'checkoutify'),
                'percentageText' => __('Percentage', 'checkoutify'),
                'flatAmountText' => __('Flat Amount', 'checkoutify'),
                'regularPriceText' => __('Regular Price', 'checkoutify'),
                'style1Text' => __('Style 1', 'checkoutify'),
                'style2Text' => __('Style 2', 'checkoutify'),
                'style3Text' => __('Style 3', 'checkoutify'),
                'style4Text' => __('Style 4', 'checkoutify'),
                'style5Text' => __('Style 5', 'checkoutify'),
                'typeToSearchProduct' => __('Type to search for a product...', 'checkoutify'),
                'nameText' => __('Name:', 'checkoutify'),
                'editOrderBumpText' => __('Edit Order Bump', 'checkoutify'),
                'deleteOrderBumpText' => __('Delete Order Bump', 'checkoutify'),
                'deleteOrderBumpMessageText' => __('Are you sure you want to delete this orderbump?', 'checkoutify'),
                'cancleText' => __('Cancel', 'checkoutify'),
                'noProductFoundText' => __('No product found', 'checkoutify'),
                'addText' => __('Add', 'checkoutify'),
                'frameTitleText' => __('Select Image', 'checkoutify'),
                'priceText' => __('Price:', 'checkoutify'),
                'saveChangesText' => __('Save Changes', 'checkoutify'),
                'updateText' => __('Update', 'checkoutify'),
            ],
            'checkoutField' => [
                'addFieldText' => __('Add Field', 'checkoutify'),
                'fieldTypeText' => __('Field Type', 'checkoutify'),
                'typeRequiredText' => __('Type is required', 'checkoutify'),
                'labelRequiredText' => __('Label is required', 'checkoutify'),
                'optionsText' => __('Options', 'checkoutify'),
                'valueRequiredText' => __('Value is required', 'checkoutify'),
                'defaultText' => __('Default', 'checkoutify'),
                'placeholderText' => __('Placeholder', 'checkoutify'),
                'validationText' => __('Validation', 'checkoutify'),
                'selectSectionText' => __('Select Section', 'checkoutify'),
                'showOnThankyouPageText' => __('Show on Thank you page', 'checkoutify'),
                'showInOrderPageText' => __('Show in Order page', 'checkoutify'),
                'requiredText' => __('Required', 'checkoutify'),
                'cancleText' => __('Cancle', 'checkoutify'),
                'addButtonText' => __('Add', 'checkoutify'),
                'nameErrorText' => __('Name is required', 'checkoutify'),
                'textText' => __('Text', 'checkoutify'),
                'emailText' => __('Email', 'checkoutify'),
                'numberText' => __('Number', 'checkoutify'),
                'textareaText' => __('Textarea', 'checkoutify'),
                'phoneText' => __('Phone', 'checkoutify'),
                'selectText' => __('Select', 'checkoutify'),
                'checkboxDropDownText' => __('Checkbox', 'checkoutify'),
                'radioText' => __('Radio', 'checkoutify'),
                'pleaseSelectText' => __('Please select', 'checkoutify'),
                'postCodeText' => __('Post Code', 'checkoutify'),
                'stateText' => __('State', 'checkoutify'),
                'billingText' => __('Billing', 'checkoutify'),
                'shippingText' => __('Shipping', 'checkoutify'),
                'addingText' => __('Adding...', 'checkoutify'),
                'nameCannotBeDuplicateText' => __('Name cannot be duplicate', 'checkoutify'),
                'editFieldText' => __('Edit Field', 'checkoutify'),
                'updatingFieldText' => __('Updating...', 'checkoutify'),
                'updateFieldText' => __('Update', 'checkoutify'),
                'deleteFieldText' => __('Delete Field', 'checkoutify'),
                'deleteFieldMessageText' => __('Are you sure you want to delete this field?', 'checkoutify'),
                'nextBtnTitleText' => __('Next', 'checkoutify'),
                'textTransformDefaultText' => __('Default', 'checkoutify'),
                'textTransformCapitalizeText' => __('Capitalize', 'checkoutify'),
                'borderStyleText' => __('Border Style', 'checkoutify'),
                'customFieldDeleteBtnText' => __('Delete', 'checkoutify'),
                'alertMessageText' => __('Successfully saved', 'checkoutify'),
                'orderBumpStyle1Text' => __('Style 1', 'checkoutify'),
                'orderBumpLabelText'  => __('Order Bump 1', 'checkoutify'),
                'productTitleText' => __('6D Screen Protector (20% OFF)', 'checkoutify'),
                'productHighlightText' => __('Special one time offer', 'checkoutify'),
                'productDescriptionText' => __('Get this scratch proof 6D Tempered Glass Screen Protector for your iPhone. Keep your phone safe and sound just like a new one.', 'checkoutify'),
                'checkboxText' => __('Grab this offer with one click!', 'checkoutify'),
                'discountTypeOriginalText' => __('Original', 'checkoutify'),
                'discountTypeApplyToText' => __('Regular Price', 'checkoutify'),
                'draftSuccessText' => __("Draft successfully saved. Please remember to add the 'Place Order' button and 'Payment' fields to your checkout.", 'checkoutify'),
                'successText' => __('Successfully saved', 'checkoutify'),
                'customFieldDeletingBtnText' => __('Deleting...', 'checkoutify'),
                'labelText' => __('Label', 'checkoutify'),
                'nameText' => __('Name', 'checkoutify'),
                'stepIncrementText' => __('Step - ', 'checkoutify'),
                'addStepText' => __('Add Step', 'checkoutify'),
                'urlLabelText' => __('URL', 'checkoutify'),
                'dateLabelText' => __('Date', 'checkoutify'),
                'startDateLabelText' => __('Start Date', 'checkoutify'),
                'endDateLabelText' => __('End Date', 'checkoutify'),
                'timeLabelText' => __('Time', 'checkoutify'),
                'startTimeLabelText' => __('Start Time', 'checkoutify'),
                'endTimeLabelText' => __('End Time', 'checkoutify'),
                'fileLabelText' => __('File', 'checkoutify'),
                'fileTypeLabelText' => __('File Type', 'checkoutify'),
                'fileSizeLabelText' => __('Maximum File Size (MB)', 'checkoutify'),
                'pdfText' => __('PDF', 'checkoutify'),
                'docxText' => __('DOCX', 'checkoutify'),
                'jpgText' => __('JPG', 'checkoutify'),
                'pngText' => __('PNG', 'checkoutify'),
                'noneText' => __('Select file types', 'checkoutify'),
                'fileTypeErrorText' => __('File types can not be empty.', 'checkoutify'),
                'xlsxText' => __('XLSX', 'checkoutify'),
                'webpText' => __('WebP', 'checkoutify'),
                'otherSectionLabelText' => __('Other', 'checkoutify'),
                'showInEmailLabelText' => __('Show In Email', 'checkoutify'),
                'editHTMLTagText' => __('Edit HTML Tags', 'checkoutify'),
                'draftMessageWithStripeRequiredFieldsText' => __("Draft saved. Add 'Place Order' and 'Payment' fields to checkout. Include all billing fields in form or enable legacy mode in Stripe settings.", 'checkoutify'),
                'draftMessageForOnlyStripeRequiredFieldsText' => __("Successfully Saved. Include all billing fields in form, or enable legacy mode in Stripe settings.", 'checkoutify'),
            ],
            'sectionModal' => [
                'addSectionText' => __('Add Section', 'checkoutify'),
                'sectionNameText' => __('Section Name', 'checkoutify'),
                'classesText' => __('Classes', 'checkoutify'),
                'classesHintsText' => __("Please don't use special characters! Classes are separated by \"space\"", 'checkoutify'),
                'columnText' => __('Column', 'checkoutify'),
                'cancleText' => __('Cancle', 'checkoutify'),
                'addButtonText' => __('Add', 'checkoutify'),
                'editSectionText' => __('Edit Section', 'checkoutify'),
                'updateButtonText' => __('Update', 'checkoutify'),
                'modeToStepText' => __('Move to step', 'checkoutify'),
                'stepSelectText' => __('Select a step', 'checkoutify'),
                'sectionDeleteHeaderText' => __('Delete Section', 'checkoutify'),
                'sectionDeleteText' => __('Delete', 'checkoutify'),
                'sectionDeleteMessageText' => __('Are you sure you want to delete this section?', 'checkoutify'),
                'sectionTitleText' => __('Section Title', 'checkoutify'),
                'sectionText' => __('Section', 'checkoutify'),
            ],
            'checkoutSingleField' => [
                'applyText' => __('Apply', 'checkoutify'),
                'snapBackCapText' => __('Snapback Cap', 'checkoutify'),
                'subTotalText' => __('Subtotal', 'checkoutify'),
                'shippingText' => __('Shipping', 'checkoutify'),
                'localPickupText' => __('Local Pickup:', 'checkoutify'),
                'totalText' => __('Total', 'checkoutify'),
                'directBankTransferText' => __('Direct Bank Transfer', 'checkoutify'),
                'bankDetailsText' => __('Make your payment directly into our bank account. Please use your Order ID as the payment reference. Your order will not be shipped until the funds have cleared in our account.', 'checkoutify'),
                'checkPaymentText' => __('Check Payment', 'checkoutify'),
                'storeNameText' => __('Please send a check to Store Name, Store Street, Store Town, Store State / County, Store Postcode.', 'checkoutify'),
                'cashOnDeliveryText' => __('Cash on Delivery', 'checkoutify'),
                'payWithCashUponDeliveryText' => __('Pay with cash upon delivery.', 'checkoutify'),
                'creditCardText' => __('Credit Card (Stripe)', 'checkoutify'),
                'payWithYourCreditCartFirstText' => __('Pay with your credit card via Stripe. TEST MODE ENABLED. In test mode, you can use the card number 4242424242424242 with any CVC and a valid expiration date or check the', 'checkoutify'),
                'payWithYourCreditCartSecondText' => __('Testing Stripe documentation', 'checkoutify'),
                'payWithYourCreditCartThirdText' => __('for more card numbers.', 'checkoutify'),
                'cardNumberText' => __('Card Number', 'checkoutify'),
                'expireDateText' => __('Expiry Date', 'checkoutify'),
                'cardCodecvcText' => __('Card Code (CVC)', 'checkoutify'),
                'savePaymentInformationText' => __('Save payment information to my account for future purchases.', 'checkoutify'),
                'userNameOrEmailText' => __('Username or email', 'checkoutify'),
                'PasswordText' => __('Password', 'checkoutify'),
                'loginText' => __('Login', 'checkoutify'),
                'rememberMeText' => __('Remember me', 'checkoutify'),
                'lostYourPasswordText' => __('Lost your password?', 'checkoutify'),
                'accountUserNameText' => __('Account username', 'checkoutify'),
                'createAccountPasswordText' => __('Create account password', 'checkoutify'),
                'enterPromoCodeText' => __('Enter Promo Code', 'checkoutify'),
                'makeItPublishToCopyText' => __('Make it publish to copy!', 'checkoutify'),
                'addPlaceOrderText' => __('Add place order and payment section to update the status', 'checkoutify'),
                'copiedText' => __('Copied!', 'checkoutify'),
            ],
            'editCheckoutRight' => [
                'newFieldText' => __('New Field', 'checkoutify'),
                'customFieldText' => __('Custom Fields', 'checkoutify'),
                'newOrderBumpText' => __('New Order Bump', 'checkoutify'),
                'createOrderBumpText' => __('Created Order Bump', 'checkoutify'),
                'noOrderBumpText' => __('No order bump found', 'checkoutify'),
                'orderBumpText' => __('Order Bumps', 'checkoutify'),
                'advancedFieldText' => __('Advanced Fields', 'checkoutify'),
                'additionalFieldText' => __('Additional Fields', 'checkoutify'),
                'shippingFieldText' => __('Shipping Fields', 'checkoutify'),
                'billingFieldText' => __('Billing Fields', 'checkoutify'),
                'allOrderBumpText' => __('All Order bump', 'checkoutify'),
                'fieldsText' => __('Fields', 'checkoutify'),
                'settingsText' => __('Settings', 'checkoutify'),
                'htmlTagsText' => __('HTML Tags', 'checkoutify'),
                'newHTMLTagsText' => __('New HTML Tag', 'checkoutify'),
                'createdHTMLTagsText' => __('No HTML tags found', 'checkoutify'),
                'deleteHTMLMessageText' => __('Are you sure you want to delete this HTML tag?', 'checkoutify'),
                'deleteHTMLText' => __('Delete HTML Tag', 'checkoutify'),
            ],
            'editCheckoutSettings' => [
                'setAsDefaultText' => __('Set as default WooCommerce checkout :', 'checkoutify'),
                'selectFunnelText' => __('Select Funnel', 'checkoutify'),
                'stepLayoutText' => __('Step Layout', 'checkoutify'),
                'singleStepText' => __('Single Step', 'checkoutify'),
                'multiStepText' => __('Multiple Steps', 'checkoutify'),
                'step1Text' => __('Steps 1', 'checkoutify'),
                'step2Text' => __('Steps 2', 'checkoutify'),
                'step3Text' => __('Steps 3', 'checkoutify'),
            ],
            'stepDeleteModal' => [
                'stepDeleteText' => __('Delete', 'checkoutify'),
                'stepDeleteMessageText' => __('Are you sure you want to delete this Step?', 'checkoutify'),
                'stepDeleteCancleText' => __('Cancel', 'checkoutify'),
            ],
            'license' => [
                'stayUpdatedText' => __('Stay Updated', 'checkoutify'),
                'updateThePluginText' => __('Update the plugin right from your WordPress Dashboard.', 'checkoutify'),
                'premiumSupportText' => __('Premium Support', 'checkoutify'),
                'premiumSupportSubText' => __('Supported by professional and courteous staff.', 'checkoutify'),
                'documentationsText' => __('Documentations', 'checkoutify'),
                'supportText' => __('Support', 'checkoutify'),
                'supportSubText' => __("Can't find solution with our documentation? Just post a ticket. Our professional team is here to solve your problems.", 'checkoutify'),
                'documentationsSubText' => __('Get started by spending some time with the documentation and generate flawless product feed for major online marketplaces within minutes.', 'checkoutify'),
                'manageLicenseText' => __('Manage License', 'checkoutify'),
                'postATicketText' => __('Post a Ticket', 'checkoutify'),
                'documentationText' => __('Documentation', 'checkoutify'),
                'deactivateText' => __('Deactivate', 'checkoutify'),
                'activateText' => __('Activate', 'checkoutify'),
            ],
            'heading' => [
                'h1LabelText' => __('H1', 'checkoutify'),
                'h2LabelText' => __('H2', 'checkoutify'),
                'h3LabelText' => __('H3', 'checkoutify'),
                'h4LabelText' => __('H4', 'checkoutify'),
                'h5LabelText' => __('H5', 'checkoutify'),
                'h6LabelText' => __('H6', 'checkoutify'),
                'spanLabelText' => __('Span', 'checkoutify'),
                'divLabelText' => __('Div', 'checkoutify'),
                'paragraphLabelText' => __('Paragraph', 'checkoutify'),
                'addHeadingLabelText' => __('Heading Type', 'checkoutify'),
                'addHtmlModalText' => __('Add HTML Tags', 'checkoutify'),
                'hiddenLabelText' => __('Hidden', 'checkoutify'),
            ]
        ];
        return $translatableStrings;
    }

    /**
     * Retrieve the Checkoutify license key.
     *
     * This method fetches the Checkoutify license key from the WordPress options table.
     *
     * @since 1.0.7
     * @return string The Checkoutify license key if it exists, otherwise an empty string.
     */
    public static function supportedFileTypes(): array
    {
        $mime_types = wp_get_mime_types();
        $file_types = [];
        $supported_file_types = ['pdf' => 'PDF', 'docx' => 'DOCX', 'jpg' => 'JPG', 'jpeg' => 'JPEG', 'png' => 'PNG'];
        foreach( $mime_types as $key => $value ){
            if(array_key_exists($key, $supported_file_types)){
                $file_types[] = array(
                    'value' => $key,
                    'label' => $supported_file_types[$key]
                );
            }
        }
        return $file_types;
    }

    /**
     * Render the script for file upload.
     *
     * This function generates the necessary script for handling file uploads.
     *
     * @param array $field The field configuration array.
     *
     * @since 1.0.7
     */
    public static function render_script_for_file_upload($field)
    {
        wp_enqueue_script( 'rex-checkoutify-custom-checkout-script', CHECKOUTIFY_URL . '/includes/Assets/Public/Js/custom-checkout.js', array( 'jquery' ), CHECKOUTIFY_VERSION, false );
        wp_localize_script('rex-checkoutify-custom-checkout-script', 'rexCheckoutifyCustomCheckout', array(
            'ajax_url' => admin_url('admin-ajax.php'),
            'nonce'    => wp_create_nonce('rex_checkoutify_custom_checkout_nonce'),
            'field_data'    => $field
        ));
    }

    /**
     * Retrieve the Checkoutify license key.
     *
     * This method fetches the Checkoutify license key from the WordPress options table.
     *
     * @since 1.0.7
     * @return string The Checkoutify license key if it exists, otherwise an empty string.
     */
    public static function getFileExtensionsString($fileTypesArray) {
        $result = [];
        foreach ($fileTypesArray as $fileType) {
            $result[] = '.' . $fileType['value'];
        }
        return implode(', ', $result);
    }

    /**
     * Retrieve the Checkoutify license key.
     *
     * This method fetches the Checkoutify license key from the WordPress options table.
     *
     * @since 1.0.7
     * @return string The Checkoutify license key if it exists, otherwise an empty string.
     */
    public static function get_customer_session_data(){
        $session= WC()->session;
        $customer_data = $session->get('customer');
        return !empty( $customer_data ) ? $customer_data : [];
    }

    /**
     * Convert an array of styles to a CSS string.
     *
     * This method takes an associative array of styles and converts it into a CSS string.
     *
     * @since 1.0.7
     *
     * @param array $styles An associative array of styles where the key is the CSS property and the value is the CSS value.
     * @return string A CSS string generated from the styles array.
     */
    public static function convertStylesToCss($styles) {
        $css = '';

        if (!empty($styles['label'])) {
            $label = $styles['label'];
            // Add basic CSS properties
            $css .= isset($label['fontSize']) ? "font-size: {$label['fontSize']}px; " : '';
            $css .= isset($label['lineHeight']) ? "line-height: {$label['lineHeight']}; " : '';
            $css .= isset($label['fontFamily']) && !empty($label['fontFamily']) ? "font-family: {$label['fontFamily']}; " : '';
            $css .= isset($label['fontWeight']) ? "font-weight: {$label['fontWeight']}; " : '';
            $css .= isset($label['color']) ? "color: {$label['color']}; " : '';
            $css .= isset($label['top']) ? "top: {$label['top']}px; " : '';
            $css .= isset($label['backgroundColor']) ? "background-color: {$label['backgroundColor']}; " : '';
            $css .= isset($label['marginBottom']) ? "margin-bottom: {$label['marginBottom']}px; " : '';

            // Add text-transform and text-decoration
            if (isset($label['textTransform']['value'])) {
                $css .= "text-transform: {$label['textTransform']['value']}; ";
            }
            if (isset($label['textDecoration']['value'])) {
                $css .= "text-decoration: {$label['textDecoration']['value']}; ";
            }
        }

        return trim($css);
    }

    /**
     * Get formatted styles by heading type.
     *
     * This method processes the provided data and style to generate
     * formatted styles specific to a heading type.
     *
     * @param array $data The data array containing style information.
     * @param string $style The style type to be formatted.
     *
     * @since 1.0.7
     * @return array The formatted styles array.
     */
    public static function getFormattedStylesByHeadingType($data, $style) {
        switch ($data['headingType']) {
            case 'p':
                $style['fontSize'] = '16';
                $style['fontWeight'] = 'normal';
                break;
            case 'span':
                $style['fontSize'] = '16';
                $style['fontWeight'] = 'normal';
                break;
            case 'h1':
                $style['fontSize'] = '32';
                $style['fontWeight'] = 'bold';
                break;
            case 'h2':
                $style['fontSize'] = '24';
                $style['fontWeight'] = 'bold';
                break;
            case 'h3':
                $style['fontSize'] = '19';
                $style['fontWeight'] = 'bold';
                break;
            case 'h4':
                $style['fontSize'] = '16';
                $style['fontWeight'] = 'bold';
                break;
            case 'h5':
                $style['fontSize'] = '13';
                $style['fontWeight'] = 'bold';
                break;
            case 'h6':
                $style['fontSize'] = '11';
                $style['fontWeight'] = 'bold';
                break;
            case 'div':
                $style['display'] = 'block';
                $style['margin'] = '0';
                $style['padding'] = '0';
                $style['border'] = 'none';
                $style['fontSize'] = '16';
                $style['fontWeight'] = 'normal';
                break;
            default:
                // Handle default case if needed
                break;
        }
        return $style;
    }


    /**
     * Check if Stripe payment gateway is active.
     *
     * @return bool True if Stripe is active, false otherwise.
     * @since 1.0.9
     */
    public static function isStripeActive() {
        if ( class_exists( 'WC_Payment_Gateways' ) ) {
            $payment_gateways = \WC_Payment_Gateways::instance();
            $gateways = $payment_gateways->get_available_payment_gateways();
            if ( isset( $gateways['stripe'] ) && defined('WC_STRIPE_VERSION') ) {
                return version_compare( WC_STRIPE_VERSION, '8.0.1', '>' );
            }
        }
        return false;
    }

    /**
     * Check if WooCommerce Blocks Checkout is active.
     *
     * @return bool True if WooCommerce Blocks Checkout is active, false otherwise.
     * @since 1.0.10
     */
    public static function is_wc_block_checkout_active() {
       return wp_script_is('wc-blocks-checkout', 'enqueued');
    }

}
