<?php

/**
 * Hook.php
 *
 * This file init classes, which ensure themes and plugins compatibility with the Checkoutify plugin.
 *
 * @package RexTheme\Checkoutify
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Compatibility;

/**
 * Class Compatibility
 *
 * This class manages Compatibility WordPress themes and plugins.
 *
 * @since 1.0.0
 */
class Compatibility {


	/**
	 * Supported plugin list
	 *
	 * @var array
	 *
	 * @since 1.0.0
	 */
	public $plugins = array();


	/**
	 * Supported theme list
	 *
	 * @var array
	 *
	 * @since 1.0.0
	 */
	public $themes = array();

	/**
	 * Compatibility constructor.
	 *
	 * Initializes the classes
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		 $this->plugins = $this->getSupportedPlugins();
		foreach ( $this->plugins as $key => $plugin ) {
			if ( isset( $plugin['class'] ) && class_exists( $plugin['class'] ) ) {
				new $plugin['class']();
			}
		}

		$this->themes = $this->getSupportedThemes();
		foreach ( $this->themes as $key => $theme ) {
			if ( isset( $theme['class'] ) ) {
				new $theme['class']();
			}
		}
	}


	/**
	 * Get supported plugins
	 *
	 * @return array array of plugins
	 * @since 1.0.0
	 */
	public function getSupportedPlugins() {
		$plugins = array();
		if ( defined( 'WPFNL_VERSION' ) ) {
			$wpf = array(
				'class' => \RexTheme\Checkoutify\Compatibility\WPFunnels::class,
				'name'  => 'WPFunnels',
			);
			array_push( $plugins, $wpf );
		}

		/**
		 * Apply a filter to allow modification or addition of supported plugins.
		 *
		 * @param array $plugins
		 *
		 * @return array $plugins
		 * @since 1.0.0
		 */
		return apply_filters( 'checkoutify_supported_plugins', $plugins );
	}


	/**
	 * Get supported themes
	 *
	 * @return array array of themes
	 * @since 1.0.0
	 */
	public function getSupportedThemes() {
		$themes = array();
		$currentTheme = wp_get_theme();

		// ---all twenty twenty themes----
		$twentyTwentyThemes = [ 'twentytwentytwo','twentytwentythree' ];

		if( in_array($currentTheme->template, $twentyTwentyThemes ) ){
			$twentyTwenty = array(
				'class' => \RexTheme\Checkoutify\Compatibility\TwentyTwenty::class,
				'name'  => 'TwentyTwenty',
			);
			array_push( $themes, $twentyTwenty );
		}

		// ----for botiga themes----
		if( 'botiga' === $currentTheme->template ){
			$botiga = array(
				'class' => \RexTheme\Checkoutify\Compatibility\Botiga::class,
				'name'  => 'Botiga',
			);
			array_push( $themes, $botiga );
		}
		
		/**
		 * blocksy support
		 * 
		 * @since 1.0.0
		 */
		if( 'blocksy' === $currentTheme->template ){
			$blocksy = array(
				'class' => \RexTheme\Checkoutify\Compatibility\Blocksy::class,
				'name'  => 'Blocksy',
			);
			array_push( $themes, $blocksy );
		}

		
		/**
		 * Apply a filter to allow modification or addition of supported themes.
		 *
		 * @param array $themes
		 *
		 * @return array $themes
		 * @since 1.0.0
		 */
		return apply_filters( 'checkoutify_supported_themes', $themes );
	}
}
