<?php
/**
 * CheckDependency Class
 *
 * @package RexTheme\Checkoutify\Dependencies
 *
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Dependencies;

/**
 * Class CheckDependency
 *
 * This class checks the dependency of the plugins activation that will be needed for the Checkoutify plugin.
 * If the necessary plugins are not active, it takes actions to deactivate Checkoutify and shows a notice.
 *
 * @package RexTheme\Checkoutify\Dependencies
 *
 * @since 1.0.0
 */
class CheckDependency {

	/**
	 * The notice message to be displayed.
	 *
	 * @var string
	 */
	public $notice;


	/**
	 * Constructor for the CheckDependency class.
	 *
	 * @param string $plugin The plugin file name to check for activation.
	 * @param string $notice The notice message to be displayed.
	 */
	public function __construct( $plugin, $notice ) {
		$this->notice  = $notice;
		$activePlugins = get_option( 'active_plugins' );
		if ( !in_array( $plugin, $activePlugins, true ) ) {
			add_action( 'admin_init', array( $this, 'deactivateCheckoutify' ) );
			add_action( 'admin_notices', array( $this, 'showNoticeToActiveWoocommerce' ) );
		}
	}

	/**
	 * Deactivate the Checkoutify plugin.
	 * This function is called when WooCommerce is not active.
	 *
	 * @since 1.0.0
	 */
	public function deactivateCheckoutify() {
		deactivate_plugins( 'checkoutify/checkoutify.php' );
	}

	/**
	 * Display a notice prompting the user to install WooCommerce to activate Checkoutify.
	 * This function is called when WooCommerce is not active.
	 *
	 * @since 1.0.0
	 */
	public function showNoticeToActiveWoocommerce() {
		echo '<div class="notice notice-warning is-dismissible">
					<p>' . esc_html__( $this->notice, 'checkoutify' ) . '</p>
				</div>';
		if ( isset( $_GET['activate'] ) ) {
			unset( $_GET['activate'] );
		}
	}
	
}


