<?php
/**
 * Checkout.php
 *
 * This file defines the Checkout class, which manages checkout-related operations in the WordPress admin panel for the Checkoutify plugin.
 * It provides methods for creating, updating, retrieving, and deleting checkouts, as well as custom input fields for the checkout process.
 *
 * @package RexTheme\Checkoutify\Module
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Module;

use \RexTheme\Checkoutify\Common\Functions;
use \RexTheme\Checkoutify\Common\Keys;
use \RexTheme\Checkoutify\Module\Field\Field;
use \RexTheme\Checkoutify\Module\Field\DefaultField;
use \RexTheme\Checkoutify\Module\Field\CustomField;
use \RexTheme\Checkoutify\Module\Style\Style;

/**
 * Class Checkout
 *
 * This class manages checkout-related operations in the WordPress admin panel for Checkoutify.
 *
 * @since 1.0.0
 */
class Checkout {

	/**
	 * Checkout Id
	 *
	 * @var int|null
	 * @since 1.0.0
	 */
	protected $id = null;

	/**
	 * Checkout constructor.
	 *
	 * Initializes a new instance of the Checkout class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
	}

	/**
	 * Create a Checkout
	 *
	 * This method creates a new checkout or updates an existing one.
	 *
	 * @param array $payload The data used to create or update the checkout.
	 *
	 * @return mixed The response from the operation.
	 *
	 * @since 1.0.0
	 */
	public function create( $payload ) {
		if ( !$payload || !is_array( $payload ) ) {
			return Functions::getErrorResponse();
		}

		$title       = !empty( $payload['title'] ) ? $payload['title'] : 'untitled';
		$postContent = !empty( $payload['postContent'] ) ? $payload['postContent'] : '';

		$checkoutId = wp_insert_post(
			apply_filters(
				'checkoutify/new_checkout_params',
				array(
					'post_type'    => CHECKOUTIFY_POST_TYPE,
					'post_title'   => wp_strip_all_tags( $title ),
					'post_content' => $postContent,
					'post_status'  => 'draft',
				)
			),
			true
		);

		$data = array(
			array(
				'label'    => __('step - 1', 'checkoutify'),
				'key'      => 'step_' . wp_rand(),
				'sections' => array(
					array(
						'label'       => __('Billing Section','checkoutify'),
						'key'         => 'billing_section_' . wp_rand(),
						'classes'     => array(
							0 => '',
						),
						'isTwoColumn' => false,
						'fields'      => array(
							'left'  => array(),
							'right' => array(),
						),
						'styles'      => array(
							'section'      => array(
								'backgroundColor'   => '#ffffff',
								'borderWidth'       => 1,
								'borderStyle'       => array(
									'title' => 'Solid',
									'value' => 'solid',
								),
								'borderColor'       => '#e5e7eb',
								'borderRadius'      => 10,
								'borderRadiusRange' => 21,
								'paddingTop'        => 14,
								'paddingRight'      => 20,
								'paddingBottom'     => 20,
								'paddingLeft'       => 20,
								'marginBottom'      => 30,
								'marginBottomRange' => 30,
							),
							'sectionTitle' => array(
								'fontSize'          => 15,
								'fontWeight'        => '600',
								'lineHeight'        => 1.3,
								'color'             => '#1b201f',
								'fontFamily'        => '',
								'textTransform'     => array(
									'title' => 'Capitalize',
									'value' => 'capitalize',
								),
								'marginBottom'      => 10,
								'marginBottomRange' => 32,
							),
						),
					),
				),
			),
		);

		$defaultFieldInstance = new DefaultField($checkoutId);
		$customFieldInstance = new CustomField($checkoutId);
		$fieldInstance = new Field($defaultFieldInstance, $customFieldInstance, $checkoutId);
		$checkoutFields = $fieldInstance->getFields();
		$billingFields  = isset( $checkoutFields['default']['billing'] ) ? $checkoutFields['default']['billing'] : [];
		$advancedFields = isset( $checkoutFields['default']['advanced'] ) ? $checkoutFields['default']['advanced'] : [];
		$mergedFields   = array_merge( $billingFields, $advancedFields );

		$defaultFieldsArray = array(
			'billing_first_name',
			'billing_last_name',
			'billing_country',
			'billing_city',
			'billing_email',
			'billing_address_1',
			'billing_state',
			'billing_postcode',
			'billing_billing',
			'payment_method',
			'place_order',
			'order_details',
            'billing_address_2',
            'billing_phone',
		);

		if(isset( $mergedFields ) && !empty( $mergedFields  ) ){
			foreach ( $mergedFields as $defaultField ) {
				if ( in_array( $defaultField['key'], $defaultFieldsArray, true ) ) {
					$data[0]['sections'][0]['fields']['left'][] = $defaultField;
				}
			}
		}

		update_post_meta( $checkoutId, Keys::CHECKOUTIFY_CHECKOUT_META, $data );

		$property = array(
			'checkoutId' => $checkoutId,
		);
		return Functions::getSuccessResponse( $property );
	}


	/**
	 * Update a single checkout with provided data.
	 *
	 * This method updates the properties of a specific checkout, such as its steps and settings, based on the provided data. It ensures that the required data is present and processes any class data within sections.
	 *
	 * @param int   $id The ID of the checkout to update.
	 * @param array $payload The data used to update the checkout, including steps and settings.
	 *
	 * @return mixed The response from the operation, indicating success or failure.
	 *
	 * @since 1.0.0
	 */
	public function updateSingleCheckout( $id, $payload ) {
		if ( !$id || !$payload || !is_array( $payload ) ) {
			$property = array(
				'type' => 'Required data is missing',
			);
			return Functions::getErrorResponse( $property );
		}

		if ( !empty( $payload['steps'] ) && is_array( $payload['steps'] ) ) {
			foreach ( $payload['steps'] as $index => $step ) {
				if ( !empty( $step['sections'] ) && is_array( $step['sections'] ) ) {
					$sections = $step['sections'];
					foreach ( $sections as $key => $section ) {
						if ( is_array( $section['classes'] ) ) {
							continue;
						}
						$explodedValue = explode( ' ', $section['classes'] );
						$payload['steps'][ $index ]['sections'][ $key ]['classes'] = $explodedValue;
					}
				}
			}
		}

		if ( !empty( $payload['steps'] ) ) {
			update_post_meta( $id, Keys::CHECKOUTIFY_CHECKOUT_META, $payload['steps'] );
		}

		if ( !empty( $payload['settings'] ) ) {
			update_post_meta( $id, Keys::CHECKOUTIFY_CHECKOUT_SETTINGS_META, $payload['settings'] );
		}

		$property = array(
			'message' => 'Data saved successfully',
		);

		return Functions::getSuccessResponse( $property );
	}

	/**
	 * Update Checkout Status
	 *
	 * This method updates the status of a checkout.
	 *
	 * @param array $payload The data used to update the checkout status.
	 *
	 * @return mixed The response from the operation.
	 *
	 * @since 1.0.0
	 */
	public function updateStatus( $payload ) {
		if ( !isset( $payload['id'], $payload['status'] ) ) {
			$property = array(
				'type' => 'Required params are missing',
			);
			return Functions::getErrorResponse( $property );
		}

		$id     = $payload['id'];
		$status = $payload['status'];

		$updatedPost = array(
			'ID'          => $id,
			'post_status' => $status,
		);

		// Update the post into the database.
		$isUpdated = wp_update_post( $updatedPost );
		if ( !$isUpdated ) {
			$property = array(
				'type' => 'Update failed',
			);
			return Functions::getErrorResponse( $property );
		}
		$property = array(
			'message' => 'Successfully updated',
		);
		return Functions::getSuccessResponse( $property );
	}

	/**
	 * Get a list of checkouts based on the provided limit.
	 *
	 * This method retrieves a list of checkout posts with details such as title, ID, status, and last modification time. It allows specifying a limit to control the number of checkouts retrieved.
	 *
	 * @param int $limit The maximum number of checkouts to retrieve. Use -1 to retrieve all checkouts.
	 *
	 * @return mixed The response containing a list of checkouts and their details.
	 *
	 * @since 1.0.0
	 */
	public function getCheckouts( $limit = -1, $status = '' ) {
        $status_array = array('publish', 'draft');
        if(!empty($status)){
            $status_array = array($status);
        }
		if ( !is_int( $limit ) ) {
			$property = array(
				'type' => 'Limit must be integer',
			);
			return Functions::getErrorResponse( $property );
		}

		$args = array(
			'post_type'      => 'rex_checkout', // Replace 'your_post_type' with the actual post type you want to query.
			'posts_per_page' => $limit, // To retrieve all posts, set this to -1. You can set it to a specific number to limit the results.
			'orderby'        => 'date', // Order by post creation date.
			'order'          => 'DESC', // Descending order (most recent first).
			'post_status'    => $status_array,
		);

		$posts = get_posts( $args );

		if ( !is_array( $posts ) ) {
			$property = array(
				'type' => 'Post not found',
			);
			return Functions::getErrorResponse( $property );
		}

		if ( is_array( $posts ) && empty( $posts ) ) {
			$property = array(
				'message' => 'Post not found',
				'posts'   => array(),
			);
			return Functions::getSuccessResponse( $property );
		}

		$post_data_array = array();
		foreach ( $posts as $post ) {
			$post_title        = $post->post_title;
			$id                = $post->ID;
			$status            = $post->post_status;
			$last_modified     = get_post_modified_time( 'U', false, $id ); // Get the last modified timestamp for the post.
			$lastUpdate        = human_time_diff( $last_modified, current_time( 'U' ) ) . ' ago'; //phpcs:ignore 
			$post_data_array[] = array(
				'title'              => $post_title,
				'id'                 => $id,
				'status'             => $status,
				'shouldUpdateStatus' => $this->shouldUpdateStatus( $id ),
				'lastUpdate'         => $lastUpdate,
			);
		}

		$property = array(
			'posts' => $post_data_array,
		);
		return Functions::getSuccessResponse( $property );
	}

	/**
	 * If a checkout has payment and place order button then allow to update status to publish
	 *
	 * @param int $id checkout id.
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function shouldUpdateStatus( $id ) {
		if ( !$id ) {
			return;
		}
		$data          = get_post_meta( $id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
		$hasPayment    = false;
		$hasPlaceOrder = false;
		if ( is_array( $data ) ) {
			foreach ( $data as $key =>$step ) {
				if ( isset( $step['sections'] ) && is_array( $step['sections'] ) ) {
					$sections = $step['sections'];
					foreach ( $sections as $key =>$section ) {
						$leftFields  = isset( $section['fields']['left'] ) ? $section['fields']['left'] : array();
						$rightFields = isset( $section['fields']['right'] ) ? $section['fields']['right'] : array();
						foreach ( $leftFields as $key => $field ) {
							if ( isset( $field['key'] ) && 'payment_method' === $field['key'] ) {
								$hasPayment = true;
							}
							if ( isset( $field['key'] ) && 'place_order' === $field['key'] ) {
								$hasPlaceOrder = true;
							}
						}
						foreach ( $leftFields as $key => $field ) {
							if ( isset( $field['key'] ) && 'payment_method' === $field['key'] ) {
								$hasPayment = true;
							}
							if ( isset( $field['key'] ) && 'place_order' === $field['key'] ) {
								$hasPlaceOrder = true;
							}
						}
					}
				}
			}
		}
		if ( $hasPlaceOrder && $hasPayment ) {
			return 'yes';
		}
		return 'no';
	}

	/**
	 * Get Single Checkout.
	 *
	 * This method retrieves a single checkout by its ID.
	 *
	 * @param int $id The ID of the checkout to retrieve.
	 *
	 * @return mixed The response containing the requested checkout.
	 *
	 * @since 1.0.0
	 */
	public function getSingleCheckout( $id ) {
		if ( !$id ) {
			$property = array(
				'type' => 'No checkout is found',
			);
			return Functions::getErrorResponse( $property );
		}
		$metaDatas = Functions::getMetaData( $id );

		$metaDatas['metas']['title'] = get_the_title( $id );

		$funnelInfo = get_post_meta( $id , Keys::CHECKOUTIFY_FUNNEL_ID, true );

		$metaDatas['metas']['funnelId'] = '';

		if( !empty( $funnelInfo ) ){
			$metaDatas['metas']['funnelId'] = $funnelInfo;
		}

		return Functions::getSuccessResponse( $metaDatas );
	}

	/**
	 * Delete a checkout by its ID.
	 *
	 * This method deletes a checkout post from the WordPress database based on the provided ID.
	 *
	 * @param int $id The ID of the checkout post to delete.
	 *
	 * @return array The response containing a success message if the checkout was deleted successfully, or an error response if the ID is missing.
	 *
	 * @since 1.0.0
	 */
	public function delete( $id ) {
		if ( !$id ) {
			$property = array(
				'type' => 'No checkout is found',
			);
			return Functions::getErrorResponse( $property );
		}
		$isDeleted = wp_delete_post( $id, true );
		if ( !$isDeleted ) {
			$property = array(
				'type' => 'Incorrect checkout id',
			);
			return Functions::getErrorResponse( $property );
		}
		$property = array(
			'message' => 'Deleted successfully',
		);
		return Functions::getSuccessResponse( $property );
	}


	/**
	 * Delete multiple checkout posts based on their IDs.
	 *
	 * This method deletes multiple checkout posts from the WordPress database based on the provided IDs. It ensures that the provided IDs are in an array format, and it iterates through the array to delete each checkout post.

	 * @param array $ids An array of IDs of the checkout posts to delete.

	 * @return mixed The response containing a success message if the checkout posts were deleted successfully, or an error response if no checkout IDs are provided.

	 * @since 1.0.0
	 */
	public function deleteMultipleCheckouts( $ids ) {
		if ( !is_array( $ids ) || ( is_array( $ids ) && empty( $ids ) ) ) {
			$property = array(
				'type' => 'No checkout is found',
			);
			return Functions::getErrorResponse( $property );
		}

		foreach ( $ids as $id ) {
			wp_delete_post( $id, true );
		}

		$property = array(
			'message' => 'Deleted successfully',
		);
		return Functions::getSuccessResponse( $property );
	}




	/**
	 * Create or update a checkout input field
	 *
	 * This method creates or updates checkout input field.
	 *
	 * @param array $data The data used to create or update checkout input field.
	 *
	 * @return mixed The response from the operation.
	 *
	 * @since 1.0.0
	 */
	public function createOrUpdateCheckoutField( $data ) {
		if ( !$data || !is_array( $data ) ) {
			$property = array(
				'type' => 'Required data is missing',
			);
			return Functions::getErrorResponse( $property );
		}
		$supportedStyles = [$data['fieldType'],'input','label'];
		$styleInstance = new Style;
		$fieldType          = !empty( $data['fieldType'] ) ? $data['fieldType'] : '';
		$label              = !empty( $data['label'] ) ? $data['label'] : '';
		$name               = !empty( $data['name'] ) ? $data['name'] : '';
		$default            = !empty( $data['default'] ) ? $data['default'] : '';
		$placeholder        = !empty( $data['placeholder'] ) ? $data['placeholder'] : '';
		$showInOrderPage    = $data['showInOrderPage'];
		$showOnThankYouPage = $data['showOnThankYouPage'];
		$showInEmail        = $data['showInEmail'];
		$required           = $data['required'];
		$validation         = !empty( $data['validation'] ) ? $data['validation'] : '';
		$options            = !empty( $data['options'] ) ? $data['options'] : array();
		$classes            = array();
		$styles             = $styleInstance->getStyles( $supportedStyles );

		$section            = !empty( $data['section'] ) ? $data['section'] : '';
		$fieldCategory      = !empty( $data['fieldCategory'] ) ? $data['fieldCategory'] : '';

        if ( !isset( $data['operationType'] ) && !isset( $data['fieldVariation'] ) && !empty($section)) {
            if('billing' === $section){
                $name = 'billing_' . $name;
            }else if('shipping' === $section){
                $name = 'shipping_' . $name;
            }else{
                $name = 'other_' . $name;
            }
        }

		if ( !empty( $data['checkoutId'] ) && 'customField' === $data['fieldCategory'] ) {
			$updateData = array(
				'fieldType'          => $fieldType,
				'label'              => $label,
				'name'               => $name,
				'key'                => $name,
				'default'            => $default,
				'placeholder'        => $placeholder,
				'showOnThankYouPage' => $showOnThankYouPage,
				'showInOrderPage'    => $showInOrderPage,
                'showInEmail'        => $showInEmail,
				'required'           => $required,
				'validation'         => $validation,
				'options'            => $options,
				'classes'            => $classes,
				'styles'             => $styles,
				'section'            => $section,
				'fieldCategory'      => $fieldCategory,
			);

            if('date' === $fieldType){
                $updateData['startDate'] = !empty( $data['startDate'] ) ? $data['startDate'] : '';
                $updateData['endDate'] =    !empty( $data['endDate'] ) ? $data['endDate'] : '';
            }

            if('time' === $fieldType){
                $updateData['startTime'] = !empty( $data['startTime'] ) ? $data['startTime'] : '';
                $updateData['endTime'] =    !empty( $data['endTime'] ) ? $data['endTime'] : '';
            }

            if('file' === $data['fieldType']){
                $updateData['maxFileSize'] = !empty( $data['maxFileSize'] ) ? $data['maxFileSize'] : '2';
            }

			$postMetaData = get_post_meta( $data['checkoutId'], Keys::CHECKOUTIFY_FIELDS, true );
			if ( !array_key_exists( 'operationType', $data ) && !isset( $data['operationType'] ) ) {
				if ( !is_array( $postMetaData ) || empty( $postMetaData ) ) {
					$postMetaData           = array();
				}
				if ( !empty( $postMetaData ) ) {
					array_push( $postMetaData, $updateData );
				} else {
					$postMetaData[] = $updateData;
				}

			} else {
				$index = array_search($name, array_column($postMetaData, 'key'));
				if( false !== $index ){
					$postMetaData[ $index ] = $updateData;
				}
			}
			update_post_meta( $data['checkoutId'], Keys::CHECKOUTIFY_FIELDS, $postMetaData );
		}elseif (!empty( $data['checkoutId'] ) && 'wooField' === $data['fieldCategory']) {
			$updateData = array(
				'label'              => $label,
				'key'                => $name,
				'required'           => $required,
			);

			$postMetaData = get_post_meta( $data['checkoutId'], Keys::CHECKOUTIFY_DEFAULT_FIELDS, true );
			if( !empty( $postMetaData ) ){
				$index = array_search($name, array_column($postMetaData, 'key'));
				if( false !== $index ){
					$postMetaData[ $index ] = $updateData;
				}
			}else{
				$postMetaData = [];
				$postMetaData[] = $updateData;
			}
			update_post_meta( $data['checkoutId'], Keys::CHECKOUTIFY_DEFAULT_FIELDS, $postMetaData );
		}

		$property = array(
			'message' 		=> 'Field created successfully',
			'customFields' 	=> 'customField' === $data['fieldCategory'] ?  $updateData : [],
		);
		return Functions::getSuccessResponse( $property );
	}

	/**
	 * Update the title field of a checkout post by its ID.
	 *
	 * This function allows you to update the title of a checkout post based on the provided data.
	 *
	 * @param array $data An array containing the update data.
	 *
	 * @return array An array with the result of the update operation.
	 *
	 * @since 1.0.0
	 */
	public function updateCheckoutTitleField( $data ) {
		if ( !$data || !is_array( $data ) || !isset( $data['id'] ) ) {
			$property = array(
				'type' => 'Required data is missing',
			);
			return Functions::getErrorResponse( $property );
		}

		$title = !empty( $data['title'] ) ? $data['title'] : 'untitled';
		$id    = !empty( $data['id'] ) ? $data['id'] : '';

		$updateData = array(
			'ID'         => $id,
			'post_title' => $title,
		);

		$isUpdated = wp_update_post( $updateData );

		if ( !$isUpdated ) {
			$property = array(
				'type' => 'Update failed',
			);
			return Functions::getErrorResponse( $property );
		}
		$property = array(
			'message' => 'Successfully updated',
		);
		return Functions::getSuccessResponse( $property );
	}

	/**
	 * Get checkout fields data
	 *
	 * This method retrieves checkout input fields by its ID.
	 *
	 * @param int $id The ID of the checkout to retrieve.
	 *
	 * @return mixed The response containing the requested checkout.
	 *
	 * @since 1.0.0
	 */
	public function getCheckoutFieldsData( $id ) {
		
		if ( !$id ) {
			$property = array(
				'type' => 'No checkout field is found',
			);
			return Functions::getErrorResponse( $property );
		}
		$customFieldInstance = new CustomField($id);
		$defaultFieldInstance = new DefaultField($id);
		$fieldInstance = new Field($defaultFieldInstance, $customFieldInstance, $id);
		$checkoutFields = $fieldInstance->getFields();
		$property  = array(
			'fieldData' => $checkoutFields,
		);
		return Functions::getSuccessResponse( $property );
	}


	/**
	 * Import a Checkout
	 *
	 * This method creates a new checkout or updates an existing one.
	 *
	 * @param array $payload The data used to create or update the checkout.
	 *
	 * @return mixed The response from the operation.
	 *
	 * @since 1.0.0
	 */
	public function import( $payload ) {
		if ( !$payload || !is_array( $payload ) ) {
			$property = array(
				'message' => 'No data found',
			);
			return Functions::getErrorResponse( $property );
		}

		$title                         = !empty( $payload['layout']['title'] ) ? $payload['layout']['title'] : 'untitled';
		$title                         = !empty( $payload['duplicate'] ) && 'yes' === $payload['duplicate'] ? $title . '-copy' : $title;
		$checkoutify_fields            = !empty( $payload['layout']['checkoutify_fields'] ) ? maybe_unserialize( $payload['layout']['checkoutify_fields'] ) : array();
		$checkoutify_checkout_data     = !empty( $payload['layout']['checkoutify_checkout_data'] ) ? maybe_unserialize( $payload['layout']['checkoutify_checkout_data'] ) : array();
		$checkoutify_checkout_settings = !empty( $payload['layout']['checkoutify_checkout_settings'] ) ? maybe_unserialize( $payload['layout']['checkoutify_checkout_settings'] ) : array();
		$checkoutId                    = wp_insert_post(
			apply_filters(
				'checkoutify/new_checkout_params',
				array(
					'post_type'   => CHECKOUTIFY_POST_TYPE,
					'post_title'  => wp_strip_all_tags( $title ),
					'post_status' => isset( $payload['status'] ) ? $payload['status'] : 'publish',
				)
			),
			true
		);
		if ( !$checkoutId ) {
			$property = array(
				'message' => 'Fail to import',
			);
			return Functions::getErrorResponse();
		}
		update_post_meta( $checkoutId, Keys::CHECKOUTIFY_CHECKOUT_META, $checkoutify_checkout_data );
		update_post_meta( $checkoutId, Keys::CHECKOUTIFY_CHECKOUT_SETTINGS_META, $checkoutify_checkout_settings );
		update_post_meta( $checkoutId, Keys::CHECKOUTIFY_FIELDS, $checkoutify_fields );

		$property = array(
			'checkoutId' => $checkoutId,
		);
		return Functions::getSuccessResponse( $property );
	}

	/**
	 * Delete a custom checkout field based on the provided data.
	 *
	 * This method retrieves the existing custom checkout fields associated with
	 * the specified checkout ID, removes the field at the given index, and updates
	 * the checkout fields meta data. It returns success or error response based on
	 * the result of the operation.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data An array containing the data needed for deleting the custom checkout field.
	 *                    Requires 'checkoutId' and 'index' keys.
	 * @return array An associative array containing a success or error message based on the operation result.
	 */
	public function deleteCustomCheckoutField( $data ) {
		if ( !is_array( $data ) || ( is_array( $data ) && empty( $data ) ) ) {
			$property = array(
				'type' => __('Data not found','checkoutify'),
			);
			return Functions::getErrorResponse( $property );
		}

		if ( !empty( $data['checkoutId'] ) ) {
			$metaData = get_post_meta( $data['checkoutId'], Keys::CHECKOUTIFY_FIELDS, true );
			if ( !empty( $metaData ) ) {
				unset( $metaData[ $data['index'] ] );
				$metaData = array_values( $metaData );
				update_post_meta( $data['checkoutId'], Keys::CHECKOUTIFY_FIELDS, $metaData );
			}
		}

		$property = array(
			'message' => 'Custom field has been deleted successfully',
		);
		return Functions::getSuccessResponse( $property );
	}


	/**
	 * Set a checkout as the default checkout.
	 *
	 * This function updates the default checkout setting based on the provided checkout ID.
	 *
	 * @param int  $checkoutId           The ID of the checkout to set as default.
	 * @param bool $maybeDefaultCheckout Whether to set the provided checkout as default ('yes') or not ('no').
	 * 
	 * @return array                    An array containing the response message.
	 * @since 1.0.0
	 */
	public function setDeatultCheckout( $checkoutId, $maybeDefaultCheckout ){
		if( !$checkoutId ){
			$property = array(
				'type' => __('Checkout id not found','checkoutify'),
			);
			return Functions::getErrorResponse( $property );
		}
		$defaultCheckout = get_option( Keys::CHECKOUTIFY_DEFAULT_CHECKOUT_KEY, false );
		if ( 'yes' === $maybeDefaultCheckout ){
			update_option( Keys::CHECKOUTIFY_DEFAULT_CHECKOUT_KEY, $checkoutId );
		}elseif ( (int)$defaultCheckout == (int)$checkoutId &&  'no' === $maybeDefaultCheckout ) {
			delete_option( Keys::CHECKOUTIFY_DEFAULT_CHECKOUT_KEY );
		}
		return Functions::getSuccessResponse();
	}


	/**
	 * Check if a given checkout is the default checkout.
	 *
	 * This function checks whether the provided checkout ID matches the default checkout ID set in the options.
	 *
	 * @param int $checkoutId The ID of the checkout to check.
	 * 
	 * @return string Returns 'yes' if the provided checkout is the default, 'no' otherwise.
	 * 
	 * @since 1.0.0
	 */
	public function maybeDeatultCheckout( $checkoutId ){
		$response = 'no';
		if( !$checkoutId ){
			return $response;
		}
		$defaultCheckout = get_option( Keys::CHECKOUTIFY_DEFAULT_CHECKOUT_KEY );
		if ( (int)$defaultCheckout == (int)$checkoutId ) {
			$response = 'yes';
		}
		return $response;
	}

	/**
	 * Set or update the funnel ID associated with a checkout in the WordPress post meta.
	 *
	 * This function allows setting or updating the funnel ID associated with a specific checkout
	 * in the WordPress post meta. It ensures that both the funnel ID and checkout ID are provided.
	 * The function returns an error response if either parameter is missing. It then updates the
	 * post meta with the new funnel ID associated with the checkout.
	 *
	 * @param string $funnelId   The ID of the funnel to associate with the checkout.
	 * @param string $checkoutId The ID of the checkout to associate with the funnel.
	 *
	 * @return array             An array containing success or error response.
	 *                           - 'type': The response type, either 'success' or 'error'.
	 *                           - 'message': The response message.
	 *
	 * @since 1.0.0 
	 */
	public function setFunnelCheckout($funnelId, $checkoutId ){

		if( !$funnelId || !$checkoutId){
			$property = array(
				'type' => __('Data not found','checkoutify'),
			);
			return Functions::getErrorResponse( $property );
		}


		$funnelInfo = get_post_meta( $checkoutId, Keys::CHECKOUTIFY_FUNNEL_ID, true );

		if( '#' === $funnelId ){
			$funnelInfo = '';
		} else{
			$funnelInfo = $funnelId;
		}

		update_post_meta( $checkoutId, Keys::CHECKOUTIFY_FUNNEL_ID, $funnelInfo );

		return Functions::getSuccessResponse();
	}

    /**
     * Create or update a checkout heading field
     *
     * This method creates or updates checkout heading field.
     *
     * @param array $data The data used to create or update checkout heading field.
     *
     * @return mixed The response from the operation.
     *
     * @since 1.0.7
     */
    public function createOrUpdateCheckoutHeadingField( $data ) {
        if( !is_array($data) || empty($data) ){
            $property = array(
                'message' => __('Data found','checkoutify'),
            );
            return Functions::getErrorResponse( $property );
        }

        $checkoutId = !empty( $data['checkoutId'] ) ? $data['checkoutId'] : '';

        $index = isset( $data['index'] ) ? $data['index'] : '';

        $html_tags = get_post_meta( $checkoutId, Keys::CHECKOUTIFY_HTML_TAGS, true );
        if( is_array($html_tags) && isset($index) && isset($html_tags[$index]) ){
            $html_tags[ $index ]['label'] = $data['label'];
        } else {
            $supportedStyles = ['label'];
            $styleInstance = new Style;
            $styles = $styleInstance->getStyles( $supportedStyles );
            $data['styles']['label'] = Functions::getFormattedStylesByHeadingType( $data, $styles['label'] );
            if( is_array($html_tags) ){
                $existingIndex = array_search($data['key'], array_column($html_tags, 'key'));
                if ($existingIndex !== false) {
                    $html_tags[$existingIndex] = $data;
                } else {
                    array_push( $html_tags, $data );
                }
            } else {
                $html_tags = [ $data ];
            }
        }
        update_post_meta( $checkoutId, Keys::CHECKOUTIFY_HTML_TAGS, $html_tags );
        $property = array(
            'message' => __('Added successfully','checkoutify'),
            'htmlTags' 	=> $data,
        );
        return Functions::getSuccessResponse( $property );
    }

    /**
     * Delete HTML tags associated with a checkout.
     *
     * This method deletes HTML tags from the checkout post meta based on the provided index.
     *
     * @param int $checkoutId The ID of the checkout.
     * @param int $index The index of the HTML tag to delete.
     *
     * @return mixed The response from the operation.
     *
     * @since 1.0.7
     */
    public function deleteHTMLTags( $checkoutId, $index ){
        if( !$checkoutId || false === $index ){
            $property = array(
                'message' => __('Data no found','checkoutify'),
            );
            return Functions::getErrorResponse( $property );
        }

        $htmlTags = get_post_meta( $checkoutId, Keys::CHECKOUTIFY_HTML_TAGS, true );
        if( !is_array($htmlTags) || !isset($htmlTags[$index]) ){
            $property = array(
                'message' => __('Data found','checkoutify'),
            );
            return Functions::getErrorResponse( $property );
        }
        unset($htmlTags[$index]);
        $htmlTags = array_values( $htmlTags );
        // $htmlTags[$index] = $data;
        update_post_meta( $checkoutId, Keys::CHECKOUTIFY_HTML_TAGS, $htmlTags );
        $property = array(
            'message' => __('Added successfully','checkoutify'),
        );
        return Functions::getSuccessResponse( $property );
    }

}


