<?php
/**
 * DefaultField.php
 *
 * Manages default fields in the checkout module.
 *
 * @package RexTheme\Checkoutify\Module\Field
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Module\Field;

use \RexTheme\Checkoutify\Common\Functions;
use \RexTheme\Checkoutify\Common\Keys;
use \RexTheme\Checkoutify\Abstracts\Fields;

/**
 * Class DefaultField
 *
 * Manages default fields in the checkout module.
 *
 * @package RexTheme\Checkoutify\Module\Field
 * @since 1.0.0
 */
class DefaultField extends Fields {

    /**
     * Section for different field categories.
     *
     * @var array
     */
    public $section = [
        'billing' => [],
        'shipping' => [],
        'additional' => [],
        'advanced' => []
    ];

    /**
     * Checkout ID.
     *
     * @var null
     */
    public $checkoutId;

    /**
     * Country instance.
     *
     * @var \WC_Countries
     */
    public $countryInstance;

    /**
     * DefaultField constructor.
     *
     * @param null $checkoutId
     * @since 1.0.0
     */
    public function __construct($checkoutId) {
        $this->checkoutId = $checkoutId;
        $this->countryInstance = new \WC_Countries();
    }

    /**
     * Get all fields.
     *
     * @return array
     * @since 1.0.0
     */
    public function getFields() {
        $this->section['billing'] = $this->getBillingFields();
        $this->section['shipping'] = $this->getShippingFields();
        $this->section['additional'] = $this->getAdditionalFields();
        $this->section['advanced'] = $this->getAdvancedFields();
        return $this->section;
    }

    /**
     * Get billing fields.
     *
     * @return array
     * @since 1.0.0
     */
    public function getBillingFields() {
        $billingFields = $this->countryInstance->get_address_fields($this->countryInstance->get_base_country(), 'billing_');
        $supportedStyles = ['label', 'input'];
        $styles = $this->getStyles($supportedStyles);
        return $this->formatFields($billingFields, $styles);
    }

    /**
     * Get shipping fields.
     *
     * @return array
     * @since 1.0.0
     */
    public function getShippingFields() {
        $shippingFields = $this->countryInstance->get_address_fields($this->countryInstance->get_base_country(), 'shipping_');
        $shippingFields['shipping_phone'] = array(
            'label' => __('Phone', 'woocommerce'),
            'required' => false,
            'class' => array('form-row-wide'),
            'validate' => array('phone'),
            'autocomplete' => 'tel',
            'section' => 'shipping',
            'priority' => 100,
        );
        $supportedStyles = ['label', 'input'];
        $styles = $this->getStyles($supportedStyles);
        return $this->formatFields($shippingFields, $styles);
    }

    /**
     * Get additional fields.
     *
     * @return array
     * @since 1.0.0
     */
    public function getAdditionalFields() {
        $supportedStyles = ['label', 'input'];
        $styles = $this->getStyles($supportedStyles);
        $isRequired = false;
        $label = __('Order notes', 'checkoutify');

        $postMetaData = get_post_meta($this->checkoutId, Keys::CHECKOUTIFY_DEFAULT_FIELDS, true);
        if (!empty($postMetaData)) {
            $index = array_search('order_comments', array_column($postMetaData, 'key'));
            if (false !== $index) {
                $isRequired = $postMetaData[$index]['required'];
                $label = $postMetaData[$index]['label'];
            }
        }

        $additionalFields = array(
            'key' => 'order_comments',
            'fieldType' => 'textarea',
            'class' => array('notes'),
            'label' => $label,
            'placeholder' => esc_attr__(
                'Notes about your order, e.g. special notes for delivery.',
                'checkoutify'
            ),
            'name' => 'order_comments',
            'required' => $isRequired,
            'enable' => 1,
            'show' => 1,
            'default' => null,
            'validate' => null,
            'delete' => 0,
            'styles' => $styles,
            'validation' => '',
            'fieldCategory' => 'wooField',
            'showOnThankYouPage' => '',
            'showInOrderPage' => '',
            'options' => array(),
        );

        return [$additionalFields];
    }

    /**
     * Format fields.
     *
     * @param array $fields
     * @param array $styles
     * @return array
     * @since 1.0.0
     */
    public function formatFields($fields = [], $styles = []) {
        $formattedField = [];
        foreach ($fields as $key => $field) {
            $isRequired = $field['required'];
            $label = $field['label'];
            $postMetaData = get_post_meta($this->checkoutId, Keys::CHECKOUTIFY_DEFAULT_FIELDS, true);
            if (!empty($postMetaData)) {
                $index = array_search($key, array_column($postMetaData, 'key'));
                if (false !== $index) {
                    $isRequired = $postMetaData[$index]['required'];
                    $label = $postMetaData[$index]['label'];
                }
            }
            $field['key'] = $key;
            $field['name'] = $key;
            $field['label'] = $label;
            $field['fieldType'] = '';
            $field['placeholder'] = '';
            $field['validation'] = '';
            $field['required'] = $isRequired;
            $field['fieldCategory'] = 'shipping_phone' !== $key ? 'wooField' : 'customField';
            $field['showOnThankYouPage'] = !('shipping_phone' !== $key);
            $field['showInOrderPage'] = !('shipping_phone' !== $key);
            $field['options'] = array();
            $field['styles'] = $styles;
            if('shipping_phone' === $key){
                $field['showInEmail'] = true;
            }
            array_push($formattedField, $field);
        }
        return $formattedField;
    }

    /**
     * Get advanced fields.
     *
     * @return array
     * @since 1.0.0
     */
    public function getAdvancedFields() {
        $fields = [
            'Payment',
            'Coupon',
            'OrderDetails',
            'Login',
            'CreateAccount',
            'PlaceOrderButton',
        ];
        /**
		 * Apply a filter to allow modification or addition of supported advance fields.
		 *
		 * @param array $fields
		 *
		 * @return array $fields
		 * @since 1.0.0
		 */
        $fields = apply_filters( 'checkoutify_supported_advance_fields', $fields );
        $advancedFields = [];
        foreach ($fields as $field) {
            $className = "\\RexTheme\\Checkoutify\\Module\Field\\" . $field;
            if ( class_exists( $className ) ) {
                $fieldInstance = new $className();
                $advancedFields[] = $fieldInstance->getField();
            }
        }
        return $advancedFields;
    }

    /**
     * Get styles for supported styles.
     *
     * @param array $supportedStyles
     * @return array
     * @since 1.0.0
     */
    public function getStyles($supportedStyles = []) {
        $styles = [];
        foreach ($supportedStyles as $supportedStyle) {
            $className = "\\RexTheme\\Checkoutify\\Module\Style\\" . ucfirst($supportedStyle);
            if( class_exists($className)){
                $styleInstance = new $className();
                $styles[$supportedStyle] = $styleInstance->getProperties();
            }
        }
        return $styles;
    }
}
