<?php
/**
 * CustomField.php
 *
 * Manages custom fields in the checkout module.
 *
 * @package RexTheme\Checkoutify\Module\Field
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Module;

use RexTheme\Checkoutify\Common\Keys;
use RexTheme\Checkoutify\Common\Functions;

/**
 * Class CustomField
 *
 * Manages custom fields in the checkout module.
 *
 * @package RexTheme\Checkoutify\Module\Field
 * @since 1.0.0
 */
class OrderBump {

    /**
     * Orderbumps array.
     *
     * @var array
     */
    public $orderbumps = [];

    /**
     * Checkout ID.
     *
     * @var null
     */
    public $checkoutId = null;

    /**
     * CustomField constructor.
     *
     * @param null $checkoutId
     * @since 1.0.0
     */
    public function __construct($checkoutId = null) {
        $this->checkoutId = $checkoutId;
    }

    
    /**
     * Get orderbumps.
     *
     * @return array
     * @since 1.0.0
     */
    public function getOrderBumps() {
        $this->orderbumps = get_post_meta( $this->checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, true );
        return $this->orderbumps;
    }

    /**
     * Create orderbump 
     * 
     * @param int $checkoutId Checkout Id
     * @return void
     * @since 1.0.0
     */
    public function createOrderBump( $checkoutId, $data ){
        if( !$checkoutId || !is_array($data) || empty($data) ){
            $property = array(
                'message' => __('Data found','checkoutify'),
            );
            return Functions::getErrorResponse( $property ); 
        }

        $bumps = get_post_meta( $checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, true );
        if( is_array($bumps) ){
            array_push( $bumps, $data );
        }else{
            $bumps = [
                $data
            ];
        }
        update_post_meta( $checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, $bumps );
        $property = array(
            'message' => __('Added successfully','checkoutify'),
        );
        return Functions::getSuccessResponse( $property ); 
    }


    /**
     * Create orderbump 
     * 
     * @param int $checkoutId Checkout Id
     * @return void
     * @since 1.0.0
     */
    public function updateOrderBump( $checkoutId, $data, $index ){
        if( !$checkoutId || !is_array($data) || empty($data) || false === $index ){
            $property = array(
                'message' => __('Data no found','checkoutify'),
            );
            return Functions::getErrorResponse( $property ); 
        }

        $bumps = get_post_meta( $checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, true );
        if( !is_array($bumps) || !isset($bumps[$index]) ){
            $property = array(
                'message' => __('Data found','checkoutify'),
            );
            return Functions::getErrorResponse( $property ); 
        }
        $bumps[$index] = $data;
        update_post_meta( $checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, $bumps );
        $property = array(
            'message' => __('Added successfully','checkoutify'),
        );
        return Functions::getSuccessResponse( $property ); 
    }


    /**
     * Create orderbump 
     * 
     * @param int $checkoutId Checkout Id
     * @return void
     * @since 1.0.0
     */
    public function deleteOrderBump( $checkoutId, $index ){
        if( !$checkoutId || false === $index ){
            $property = array(
                'message' => __('Data no found','checkoutify'),
            );
            return Functions::getErrorResponse( $property ); 
        }

        $bumps = get_post_meta( $checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, true );
        if( !is_array($bumps) || !isset($bumps[$index]) ){
            $property = array(
                'message' => __('Data found','checkoutify'),
            );
            return Functions::getErrorResponse( $property ); 
        }
        unset($bumps[$index]);
        $bumps = array_values( $bumps );
        // $bumps[$index] = $data;
        update_post_meta( $checkoutId, Keys::CHECKOUTIFY_ORDERBUMP, $bumps );
        $property = array(
            'message' => __('Added successfully','checkoutify'),
        );
        return Functions::getSuccessResponse( $property ); 
    }


    /**
     * Get orderbump product
     * 
     * @param string @searchItem
     * @return array product array
     * @since 1.0.0
     */
    public function getObProduct( $srerchIteam ){
        $products        = [];
        $dataStore = \WC_Data_Store::load('product');
        $ids        = $dataStore->search_products($srerchIteam, '', false, false, 10);
      
        $productObjects = array_map(
            function ($id) {
                $product = wc_get_product($id);
                return $product;
            },
            $ids
        );

        if( !is_array($productObjects) ){
            $property = array(
                'message' => __('No product found','checkoutify'),
                'products' => [],
            );
            return Functions::getErrorResponse( $property );
        }

        foreach ($productObjects as $productObject) {
            if( $productObject && (( $productObject->managing_stock() && $productObject->get_stock_quantity() > 0 ) || ( !$productObject->managing_stock() && 'outofstock' !== $productObject->get_stock_status() )) ){
                $formattedName = $productObject->get_name();
                if( 'variable' === $productObject->get_type() ||  'variable-subscription' === $productObject->get_type() ) {
                    if( is_plugin_active( 'woocommerce-subscriptions/woocommerce-subscriptions.php' ) ){
                        $signUpFee = \WC_Subscriptions_Product::get_sign_up_fee( $productObject );
                    }
                    $variations = $productObject->get_available_variations();
                    if( is_array($variations ) ){
                        foreach ($variations as $variation) {
                            $variationProduct = wc_get_product($variation['variation_id']);
                            if( $variationProduct ){
                                error_log(print_r($variationProduct, true));
                                if( ( $variationProduct->managing_stock() && $variationProduct->get_stock_quantity() > 0 ) || ( !$variationProduct->managing_stock() && $variationProduct->get_stock_status() !== 'outofstock' ) ){
                                    $products[] = [
                                        'name'  =>   $this->gegFormatedProductName( $variationProduct ),
                                        'id'    => $variation['variation_id'],
                                        'regularPrice' =>  $variation['display_regular_price'],
                                        'salePrice'    => $variationProduct->get_sale_price() ?: '',
                                        'description' => $productObject->get_short_description(),
                                        'img' =>  wp_get_attachment_image_url( $productObject->get_image_id(), 'full' ),
                                    ];
                                }
                            }
                        }
                    }

                }else {
                    $products[] = [
                        'name' => rawurldecode($formattedName),
                        'id'    => $productObject->get_id(),
                        'regularPrice' => $productObject->get_regular_price(),
                        'salePrice' => $productObject->get_sale_price(),
                        'description' => $productObject->get_short_description(),
                        'img' =>  wp_get_attachment_image_url( $productObject->get_image_id(), 'full' ) ,
                    ];
                }
            }
        }
        $property = array(
            'message' => __('Product found','checkoutify'),
            'products' => $products,
        );
        return Functions::getSuccessResponse( $property );
    }



    /**
     * Get formated product name
     *
	 * @param Object $product
	 * 
     * @return String
     */
    public static function gegFormatedProductName( $product ){
		$eachChildAttr = [];
		$title = '';

		if( !$product ){
            return $title;
        }

        if( 'variable' === $product->get_type()  || 'variation' === $product->get_type() || 'subscription_variation' === $product->get_type() || 'variable-subscription' === $product->get_type() ){
            $attrSummary = $product->get_attribute_summary();
            $attrArray   = explode( ",", $attrSummary );
            foreach ( $attrArray as $attribute ) {
                $attr              = strpbrk( $attribute, ":" );
                $eachChildAttr[] = $attr;
            }
        }

        if( $eachChildAttr ){
            $eachChildFormattedAttr = [];
            foreach ( $eachChildAttr as $attribute ) {
                $eachChildFormattedAttr[] = str_replace( ": ", " ", $attribute );
            }
            $title = $product->get_title() . " - ";
            $title = $title . implode( ', ', $eachChildFormattedAttr );
        }else{
            $title = $product->get_title();
        }

        return $title;
    }

}
