<?php

/**
 * Hook.php
 *
 * This file defines the Hook class, which manages hooks and actions for the WordPress frontend related to the Checkoutify plugin.
 * It provides functionality to relocate WooCommerce template files to allow custom templates from the Checkoutify plugin directory to be used in place of the default WooCommerce templates.
 *
 * @package RexTheme\Checkoutify\Frontend
 * @since 1.0.0
 */

namespace RexTheme\Checkoutify\Frontend;

use Error;
use RexTheme\Checkoutify\Common\Keys;

use RexTheme\Checkoutify\Common\Functions;
/**
 * Class Hook
 *
 * This class manages hooks and actions for the WordPress frontend related to Checkoutify.
 *
 * @since 1.0.0
 */
class Hook {


	/**
	 * Hook constructor.
	 *
	 * Initializes the class and adds necessary actions for the WordPress frontend.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// Add a filter to relocate WooCommerce templates.
		if ( !is_admin() ) {
			add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
			add_action( 'woocommerce_checkout_process', array( $this, 'validateCustomCheckoutFields' ) );
			add_action( 'woocommerce_checkout_process', array( $this, 'resetCookieForMainProducts' ) );
			add_action( 'woocommerce_after_checkout_validation', array( $this, 'removeDefaultCheckoutValidation' ), 10, 2 );
			add_filter( 'the_content', array( $this, 'replaceBlockContent' ), 9999 );
			add_filter( 'woocommerce_locate_template', array( $this, 'relocateWoocommerceTemplate' ), 9999, 3 );
		}
		add_action( 'wp_ajax_update_states', array( $this, 'updateStatesCallback' ) );
		add_action( 'wp_ajax_nopriv_update_states', array( $this, 'updateStatesCallback' ) );

		add_action( 'wp_ajax_checkoutify_apply_coupon', array( $this, 'checkoutifyApplyCoupon' ) );
		add_action( 'wp_ajax_nopriv_checkoutify_apply_coupon', array( $this, 'checkoutifyApplyCoupon' ) );
		add_action( 'wp_ajax_checkoutify_apply_orderbump', array( $this, 'checkoutifyApplyOrderbump' ) );
		add_action( 'wp_ajax_nopriv_checkoutify_apply_orderbump', array( $this, 'checkoutifyApplyOrderbump' ) );

		add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'updaterOrderMetaByCustomField' ), 9999 );
		add_action( 'woocommerce_admin_order_data_after_billing_address', array( $this, 'showCustomFieldInOrderDetailsBilling' ) );
		add_action( 'woocommerce_admin_order_data_after_shipping_address', array( $this, 'showCustomFieldInOrderDetailsShipping' ) );
		add_filter( 'woocommerce_order_get_formatted_billing_address', array( $this, 'checkoutifyDisplayExtraFieldValueInBilling' ), 9999, 3 );
		add_filter( 'woocommerce_order_get_formatted_shipping_address', array( $this, 'checkoutifyDisplayExtraFieldValueInShipping' ), 9999, 3 );

        add_action( 'wp_ajax_checkoutify_remove_coupon', array( $this, 'checkoutifyRemoveCoupon' ) );
        add_action( 'wp_ajax_nopriv_checkoutify_remove_coupon', array( $this, 'checkoutifyRemoveCoupon' ) );

		add_action( 'admin_notices', array( $this, 'checkoutifyNotice' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueueMediaUploaderScripts' ) );

		add_action( 'wp', array( $this, 'setCartItemCookies' ), 55 );

		/**
		 * Remove Avada hook
		 *
		 * @since 1.0.0
		 */
		$currentTheme = wp_get_theme();
		if ( 'Avada' === $currentTheme->template ) {
			add_action( 'wp', array( $this, 'removeAvadaHooks' ), 100 );
		}

        add_action('wp_ajax_upload_custom_file', array($this,'handle_custom_file_upload'));
        add_action('wp_ajax_nopriv_upload_custom_file',array( $this,'handle_custom_file_upload'));
        add_filter('woocommerce_email_attachments', array( $this,'attach_custom_file_to_email'),10,3);


        add_action( 'woocommerce_order_details_after_order_table', array($this, 'display_custom_field_in_thankyou_page'), 20 );
        add_action( 'woocommerce_admin_order_data_after_billing_address', array($this, 'display_custom_field_in_order_details_page' ) );
        add_action('woocommerce_email_customer_details', array($this, 'display_custom_field_in_email_page'), 10, 4);

        add_action('woocommerce_before_calculate_totals', array($this, 'adjust_order_bump_price') );
    }

	/**
	 * Adjust the price of order bump items in the cart.
	 *
	 * This function checks if the current context is not an admin page and if the
	 * 'woocommerce_before_calculate_totals' action has not been triggered more than once.
	 * It then iterates through the cart items and applies the custom price to items
	 * marked as order bumps.
	 *
	 * @param \WC_Cart $cart The WooCommerce cart object.
	 */
	public function adjust_order_bump_price( \WC_Cart $cart ) {
		if ( is_admin() && ! defined( 'DOING_AJAX' ) ) return;

		foreach ( $cart->get_cart() as $cart_item ) {
			if ( !empty( $cart_item['checkoutify_order_bump_custom_price'] ) && true  === $cart_item['checkoutify_order_bump_custom_price'] ) {
				// Apply discount or sale price
				$cart_item['data']->set_price( $cart_item['custom_price'] );
			}
		}
	}

    /**
     * Handle custom file upload via AJAX.
     *
     * This function processes the uploaded file, moves it to the WooCommerce uploads directory,
     * and returns the file URL if the upload is successful.
     *
     * @since 1.0.7
     */
    function handle_custom_file_upload() {
        check_ajax_referer('rex_checkoutify_custom_checkout_nonce', 'nonce');

        if (!empty($_FILES['file']['name'])) {
            $file_name = $_FILES['file']['name'];
            $file_tmp = $_FILES['file']['tmp_name'];


            $filetype = wp_check_filetype($file_name);
            $allowed_mime_types = get_allowed_mime_types();
            $wp_max_file_size = wp_max_upload_size();
            $selected_file_size = filesize($file_tmp);
            $admin_selected_file_size = !empty($_POST['max_file_size']) ? sanitize_text_field( $_POST['max_file_size'] ) : $wp_max_file_size;
            $required_file_size = 1024 * 1024 * (int)$admin_selected_file_size;

            if ($selected_file_size > $wp_max_file_size) {
                wp_send_json_error(array( 'message' => __('WordPress file size exceeds the limit.', 'checkoutify')));
            }

            if ($selected_file_size > $required_file_size) {
                wp_send_json_error(array( 'message' => 'File size exceeds the limit of '.$admin_selected_file_size.' MB.'));
            }

            if (in_array($filetype['type'], $allowed_mime_types, true)) {
                $upload_dir = wp_upload_dir();
                $woocommerce_uploads_dir = $upload_dir['basedir'] . '/woocommerce_uploads';

                if (!file_exists($woocommerce_uploads_dir)) {
                    wp_mkdir_p($woocommerce_uploads_dir);
                }

                $destination = $woocommerce_uploads_dir . '/' . $file_name;
                $movefile = move_uploaded_file($file_tmp, $destination);

                if ($movefile) {
                    $file_url = $upload_dir['baseurl'] . '/woocommerce_uploads/' . $file_name;
                    wp_send_json_success(array('url' => esc_url($file_url)));
                } else {
                    wp_send_json_error(array( 'message' => __('Failed to move the uploaded file.', 'checkoutify') ) );
                }
            } else {
                wp_send_json_error(array( 'message' =>__('Invalid file type.','checkoutify') ) );
            }
        } else {
            wp_send_json_error(array( 'message' => __('No file uploaded.', 'checkoutify') ) );
        }

    }

    /**
     * Attach custom file to WooCommerce email.
     *
     * This function attaches the custom file uploaded during checkout to specific WooCommerce emails.
     * The file URL is retrieved from the order meta and added to the email attachments.
     *
     * @since 1.0.7
     *
     * @param array    $attachments The current array of email attachments.
     * @param string   $email_id    The ID of the WooCommerce email being sent.
     * @param WC_Order $order       The WooCommerce order object.
     * @return array   The modified array of email attachments.
     */
    function attach_custom_file_to_email($attachments, $email_id, $order) {
        if ($email_id === 'new_order' || $email_id === 'customer_processing_order' || $email_id === 'customer_completed_order') {
            $file_url = get_post_meta($order->get_id(), '_custom_file_field', true);
            if ($file_url) {
                $upload_dir = wp_upload_dir();
                $file_path = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $file_url);
                $attachments[] = $file_path;
            }
        }
        return $attachments;
    }


	/**
	 * Replace wc block checkout
	 *
	 * @return string $content
	 * @since 1.0.0
	 */
	public function replaceBlockContent( $content ) {
		$defaultCheckout = Functions::getDefaultCheckout();
		if ( $defaultCheckout && strpos( $content, 'wp-block-woocommerce-checkout' ) !== false ) {
			$content = do_shortcode( '[woocommerce_checkout]' );
		}
		return $content;
	}

	/**
	 * Remove custom markup for Adava theme by removing hooks
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function removeAvadaHooks() {
		global $avada_woocommerce;
		remove_action( 'woocommerce_before_checkout_form', array( $avada_woocommerce, 'before_checkout_form' ) );
		remove_action( 'woocommerce_before_checkout_form', array( $avada_woocommerce, 'checkout_coupon_form' ) );
	}


	/**
	 * Update state field type based on selected country
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public function updateStatesCallback() {
		check_ajax_referer( 'checkout_nonce', 'security' );
		$country = sanitize_text_field( $_POST['country'] );
		$section = sanitize_text_field( $_POST['section'] );
		$states  = WC()->countries->get_states( $country );
		$html    = '';
		$isState = true;
		if ( !is_null( $country ) && is_array( $states ) && !empty( $states ) ) {
			// $html .= '<select name="'.$section.'_state" id="'.$section.'"_state" class="rex-checkoutify-input state_select checkoutify-state-select">';

			foreach ( $states as $ckey => $cvalue ) {
				$html .= '<option value="' . esc_attr( $ckey ) . '" ' . selected( null, $ckey, false ) . '>' . esc_html( $cvalue ) . '</option>';
			}
				// $html .= '</select>';
		} else {
			$html   .= '<p id="' . $section . '_state_field">';
			$isState = false;
			$html   .= '<input type="text" class="rex-checkoutify-input checkoutify-state" value=""  placeholder="" name="' . esc_attr( $section . '_state' ) . '" id="' . esc_attr( $section . '_state' ) . '"data-input-classes=""/>';
			$html   .= '</p>';
		}

		wp_send_json(
			array(
				'success' => true,
				'html'    => $html,
				'isState' => $isState,
			)
		);
	}

	/**
	 * Relocate WooCommerce template files.
	 *
	 * This function checks if a custom template exists in the Checkoutify plugin directory and uses it instead of the default WooCommerce template.
	 *
	 * @param string $template The template file.
	 * @param string $templateName The name of the template.
	 * @param string $templatePath The path to the template.
	 * @return string The relocated template path.
	 *
	 * @since 1.0.0
	 */
	public function relocateWoocommerceTemplate( $template, $templateName, $templatePath ) {

		// Store the original template path.
		$_template  = $template;
		$pluginPath = CHECKOUTIFY_DIR . '/woocommerce/templates/';
		global $post;
		$stepId     = isset( $post->ID ) ? $post->ID : '';
		$funnelId   = get_post_meta( $stepId, '_funnel_id', true );
		$checkoutId = Functions::maybeCheckoutAvailableByFunnelId( $funnelId );

		// Check if the custom shortcode is present.
		$custom_shortcode_present = Functions::isCheckoutifyShortcodeAvailable(); // Replace with your actual check.
		$defaultCheckout          = Functions::getDefaultCheckout();

		// Check if a custom template exists in the plugin directory.
		if ( ( ( $custom_shortcode_present || $defaultCheckout ) || ( $checkoutId ) ) && file_exists( $pluginPath . $templateName ) ) {
			$template = $pluginPath . $templateName;
		}

		// If the template is not found, revert to the original path.
		if ( !$template ) {
			$template = $_template;
		}

		return $template;
	}


	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_scripts() {
		if ( class_exists( 'WooCommerce' ) ) {
			wp_enqueue_script( 'wc-checkout' );
			wp_enqueue_script( 'wc-country-select' );
			wp_enqueue_script( 'wc-address-i18n' );
		}
		wp_enqueue_script( 'checkoutify-public', CHECKOUTIFY_URL . '/includes/Assets/Public/Js/Main.js', array( 'jquery' ), CHECKOUTIFY_VERSION, false );
		wp_localize_script(
			'checkoutify-public',
			'checkoutifyPublicObj',
			array(
				'ajaxurl'           => admin_url( 'admin-ajax.php' ),
				'is_user_logged_in' => is_user_logged_in(),
				'user_id'           => get_current_user_id(),
				'security'          => wp_create_nonce( 'checkout_nonce' ),
                'customer_session'  => Functions::get_customer_session_data(),
                'is_wc_block_checkout' => Functions::is_wc_block_checkout_active(),
			)
		);
	}

	public function validateCustomCheckoutFields() {
		if ( isset( $_POST['checkoutify_id'] ) ) {
			$instance       = new \RexTheme\Checkoutify\Module\Checkout();
			$data           = $instance->getSingleCheckout( sanitize_text_field( $_POST['checkoutify_id'] ) );
			$checkoutFields = array();

			if ( is_array( $data ) && isset( $data['metas']['checkoutify_checkout_data'] ) ) {
				$checkoutInformation = $data['metas']['checkoutify_checkout_data'];

				if ( empty( $checkoutInformation ) && !is_array( $checkoutInformation ) ) {
					return;
				}

				foreach ( $checkoutInformation as $checkoutStepData ) {
					$checkoutSections = $checkoutStepData['sections'];
					if ( !empty( $checkoutSections ) && is_array( $checkoutSections ) ) {
						foreach ( $checkoutSections as $sectionData ) {
							$checkoutFieldsLeft  = $sectionData['fields']['left'];
							$checkoutFieldsRight = $sectionData['fields']['right'];
							$checkoutFields      = array_merge( $checkoutFields, $checkoutFieldsLeft, $checkoutFieldsRight );
						}
					}
				}
			}

			if ( empty( $checkoutFields ) && !is_array( $checkoutFields ) ) {
				return;
			}

			foreach ( $checkoutFields as $checkoutField ) {
				if ( isset( $checkoutField['required'] ) && $checkoutField['required'] ) {
					if ( isset( $_POST[ $checkoutField['key'] ] ) && !$_POST[ $checkoutField['key'] ] ) {
						wc_add_notice( __( $checkoutField['label'] . ' is a required field.' ), 'error' );
					}
				}

                if ( !empty( $checkoutField['fieldType'] ) && 'time' === $checkoutField['fieldType'] && !empty( sanitize_text_field( wp_unslash( $_POST[ $checkoutField['key'] ] ) ) ) ) {
                    $time = sanitize_text_field( wp_unslash( $_POST[ $checkoutField['key'] ] ) );
                    $start_time = !empty( $checkoutField['startTime'] ) ? $checkoutField['startTime'] : '00:00';
                    $end_time = !empty( $checkoutField['endTime'] ) ? $checkoutField['endTime'] : '23:59';
                    if ( strtotime( $time ) < strtotime( $start_time ) || strtotime( $time ) > strtotime( $end_time ) ) {
                        wc_add_notice( __( $checkoutField['label'] . ' must be between '.$start_time.' and '.$end_time. '.' ), 'error' );
                    }
                }

                if ( !empty( $checkoutField['fieldType'] ) && 'url' === $checkoutField['fieldType'] && !empty( sanitize_text_field( wp_unslash( $_POST[ $checkoutField['key'] ] ) ) ) ) {
                    $url = sanitize_text_field( wp_unslash( $_POST[ $checkoutField['key'] ] ) );
                    if ( !filter_var( $url, FILTER_VALIDATE_URL ) ) {
                        wc_add_notice( __( $checkoutField['label'] . ' is not a valid URL.' ), 'error' );
                    }
                }

                if(!empty( $checkoutField['fieldType'] ) && 'file' === $checkoutField['fieldType'] && empty( sanitize_url( $_POST['custom_file_url'] ) ) ){
                    wc_add_notice( __( $checkoutField['label'] . ' is required.' ), 'error' );
                }
			}
		}
	}


    /**
     * Applies a coupon to the WooCommerce cart via AJAX.
     *
     * This function checks for a valid nonce, verifies that a coupon code is provided,
     * and ensures the coupon hasn't already been applied to the cart before applying it.
     * A JSON response with success or failure is returned to the client.
     *
     * @since 1.0.0
     *
     * @return void
     */
	public function checkoutifyApplyCoupon() {
		check_ajax_referer( 'checkout_nonce', 'security' );
		if ( !WC()->cart ) {
			wp_send_json(
				array(
					'success' => false,
					'message' => __('Cart is empty', 'checkoutify'),
				)
			);
		}

		if ( !isset( $_POST['code'] ) ) {
			wp_send_json(
				array(
					'success' => false,
					'message' => __('Please enter coupon code', 'checkoutify'),
				)
			);
		}

		$code = sanitize_text_field( $_POST['code'] );
		if ( WC()->cart->has_discount( $code ) ) {
			wp_send_json(
				array(
					'success' => false,
					'message' => __('Coupon has already been applied', 'checkoutify'),
				)
			);
		}
		WC()->cart->apply_coupon( $code );
		wp_send_json(
			array(
				'success' => true,
				'message' => __('Coupon has applied successfully', 'checkoutify'),
			)
		);
	}

	/**
	 * Display custom fields in the WooCommerce order details on the admin page.
	 *
	 * Retrieves and displays specific custom fields associated with the order from the
	 * related checkout information. The fields are determined by the 'checkoutify_checkout_data'
	 * meta stored in the order's associated checkout ID.
	 *
	 * @since 1.0.0
	 *
	 * @param WC_Order $order The WooCommerce order object.
	 * @return void
	 */
	public function showCustomFieldInOrderDetailsBilling( $order ) {
		$checkout_id         = $order->get_meta( 'checkout_id' );
		$checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
		$checkoutFields      = $this->formatedInputFields( $checkoutInformation );

		if ( empty( $checkoutFields ) ) {
			return;
		}


		foreach ( $checkoutFields as $index => $field ) {
			if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showInOrderPage'] ) && $field['showInOrderPage'] && isset( $field['section'] ) && 'billing' === $field['section'] ) {
				$orderMetaData = $order->get_meta( $field['name'] );
				if ( !empty( $orderMetaData ) ) {
					echo '<p><strong>' . __( $field['label'] ) . ':</strong> ' . $orderMetaData . '</p>';
				}
			}

            if ( isset( $field['name'], $field['fieldType'], $field['headingType'] ) && 'heading' === $field['fieldType'] && 'hidden' === $field['headingType']) {
                $orderMetaData = $order->get_meta( $field['name'] );
                if ( !empty( $orderMetaData ) ) {
                    echo '<p><strong>' . __( $field['label'] ) . ':</strong> ' . $orderMetaData . '</p>';
                }
            }

            if(!empty($field['fieldType']) && 'file' === $field['fieldType'] && isset( $field['section'] ) && 'billing' === $field['section']) {
                $file_url = $order->get_meta( '_custom_file_field' );
                if ($file_url) {
                    echo '<p><strong>' . __('Uploaded File') . ':</strong> <a href="' . esc_url($file_url) . '" target="_blank">' . __('View File') . '</a></p>';
                }
            }
		}
	}

	/**
	 * Display custom fields in the WooCommerce order details on the admin page.
	 *
	 * Retrieves and displays specific custom fields associated with the order from the
	 * related checkout information. The fields are determined by the 'checkoutify_checkout_data'
	 * meta stored in the order's associated checkout ID.
	 *
	 * @since 1.0.0
	 *
	 * @param WC_Order $order The WooCommerce order object.
	 * @return void
	 */
	public function showCustomFieldInOrderDetailsShipping( $order ) {
		$checkout_id         = $order->get_meta( 'checkout_id' );
		$checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
		$checkoutFields      = $this->formatedInputFields( $checkoutInformation );

		if ( empty( $checkoutFields ) ) {
			return;
		}

		foreach ( $checkoutFields as $index => $field ) {

			if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showInOrderPage'] ) && $field['showInOrderPage'] && isset( $field['section'] ) && 'shipping' === $field['section'] && isset( $field['name'] ) && 'shipping_phone' !== $field['name']) {
				$orderMetaData = $order->get_meta( $field['name'] );
				if ( !empty( $orderMetaData ) ) {
					echo '<p><strong>' . __( $field['label'] ) . ':</strong> ' . $orderMetaData . '</p>';
				}
			}
            if(!empty($field['fieldType']) && 'file' === $field['fieldType'] && isset( $field['section'] ) && 'shipping' === $field['section']) {
                $file_url = $order->get_meta( '_custom_file_field' );
                if ($file_url) {
                    echo '<p><strong>' . __('Uploaded File') . ':</strong> <a href="' . esc_url($file_url) . '" target="_blank">' . __('View File') . '</a></p>';
                }
            }
		}
	}

	/**
	 * Update order meta based on custom fields submitted during the checkout process.
	 *
	 * Retrieves custom fields information associated with the specified checkout ID
	 * and updates the corresponding order meta data. Only custom fields of the 'customField'
	 * category are considered for updating.
	 *
	 * @since 1.0.0
	 *
	 * @param int $order_id The ID of the WooCommerce order to update.
	 * @return void
	 */
	public function updaterOrderMetaByCustomField( $order_id ) {
		if ( !isset( $_POST['checkoutify_id'] ) ) {
			return;
		}
		$checkout_id         = sanitize_text_field( $_POST['checkoutify_id'] );
		$checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
		$checkoutFields      = $this->formatedInputFields( $checkoutInformation );
		$order               = wc_get_order( $order_id );
		if ( empty( $checkoutFields ) && !is_array( $checkoutFields ) ) {
			return;
		}

		foreach ( $checkoutFields as $checkoutField ) {
			if ( isset( $checkoutField['name'], $_POST[ $checkoutField['name'] ], $checkoutField['fieldCategory'] ) && 'customField' === $checkoutField['fieldCategory'] ) {
				$order->update_meta_data( $checkoutField['name'], sanitize_text_field( $_POST[ $checkoutField['name'] ] ), true );
				$order->update_meta_data( 'checkout_id', $checkout_id, true );
			}

            if ( isset( $checkoutField['name'], $_POST[ $checkoutField['name'] ], $checkoutField['fieldType'], $checkoutField['headingType'] ) && 'heading' === $checkoutField['fieldType'] && 'hidden' === $checkoutField['headingType']) {
                $order->update_meta_data( $checkoutField['name'], sanitize_text_field( $_POST[ $checkoutField['name'] ] ), true );
                $order->update_meta_data( 'checkout_id', $checkout_id, true );
            }

            if (isset($_POST['custom_file_url'])) {
                if (!empty($checkoutField['fieldType']) && 'file' === $checkoutField['fieldType'] && !empty(sanitize_text_field($_POST['custom_file_url']))) {
                    $order->update_meta_data('_custom_file_field', $_POST['custom_file_url'], true);
                    $order->update_meta_data( 'checkout_id', $checkout_id, true );
                }
            }
		}

		$order->save();
	}

	/**
	 * Format and retrieve input fields from the provided checkout information.
	 *
	 * This function takes an array of checkout information, extracts input fields
	 * from the sections, and returns a formatted array of checkout fields.
	 *
	 * @since 1.0.0
	 *
	 * @param array $checkoutInformation An array containing checkout information.
	 * @return array An array of formatted checkout fields.
	 */
	public function formatedInputFields( $checkoutInformation ) {
		$checkoutFields = array();

		if ( empty( $checkoutInformation ) && !is_array( $checkoutInformation ) ) {
			return $checkoutFields;
		}

		foreach ( $checkoutInformation as $checkoutStepData ) {
			$checkoutSections = $checkoutStepData['sections'];

			if ( !empty( $checkoutSections ) && is_array( $checkoutSections ) ) {
				foreach ( $checkoutSections as $sectionData ) {
					$checkoutFieldsLeft  = $sectionData['fields']['left'];
					$checkoutFieldsRight = $sectionData['fields']['right'];
					$checkoutFields      = array_merge( $checkoutFields, $checkoutFieldsLeft, $checkoutFieldsRight );
				}
			}
		}

		return $checkoutFields;
	}

	/**
	 * Display extra field value in the WooCommerce billing address.
	 *
	 * Appends the custom field values to the WooCommerce billing address. The custom
	 * field values are determined by the 'checkoutify_checkout_data' meta stored in the
	 * order's associated checkout ID. Only custom fields of the 'customField' category
	 * with 'showOnThankYouPage' set to true are considered for display.
	 *
	 * @since 1.0.0
	 *
	 * @param string   $address The current billing address HTML string.
	 * @param string   $rawAddress The raw billing address string.
	 * @param WC_Order $order The WooCommerce order object.
	 * @return string The modified billing address HTML string with custom field values.
	 */
	public function checkoutifyDisplayExtraFieldValueInBilling( $address, $rawAddress, $order ) {
		$address .= $this->addBillingCustomFieldValueInThankyou( $address, $order );
		return $address;
	}

	/**
	 * Add custom billing field value to the WooCommerce thank you page.
	 *
	 * Retrieves and formats specific custom billing fields associated with the order
	 * from the related checkout information. The fields are determined by the 'checkoutify_checkout_data'
	 * meta stored in the order's associated checkout ID. Only custom fields of the 'customField'
	 * category with 'showOnThankYouPage' set to true are considered for display.
	 *
	 * @since 1.0.0
	 *
	 * @param string   $address The current billing address HTML string.
	 * @param WC_Order $order The WooCommerce order object.
	 * @return string The modified billing address HTML string with custom field values.
	 */
	private function addBillingCustomFieldValueInThankyou( $address, $order ) {
		$checkout_id         = $order->get_meta( 'checkout_id' );
		$checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
		$customAddress       = '';
		$checkoutFields      = $this->formatedInputFields( $checkoutInformation );
		if ( empty( $checkoutFields ) ) {
			return $customAddress;
		}

		foreach ( $checkoutFields as $index => $field ) {
			if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showOnThankYouPage'] ) && $field['showOnThankYouPage'] && isset( $field['section'] ) && 'billing' === $field['section'] ) {
				$orderMetaData = $order->get_meta( $field['name'] );
				if ( !empty( $orderMetaData ) ) {
					$customAddress .= '<br/>' . $orderMetaData . '<br/>';
				}
			}
            if(!empty($field['fieldType']) && 'file' === $field['fieldType'] && 'billing' === $field['section'] ) {
                $file_url = $order->get_meta( '_custom_file_field' );
                if ($file_url) {
                    $customAddress .= '<br/><p><strong>' . __('Uploaded File') . ':</strong> <a href="' . esc_url($file_url) . '" target="_blank">' . __('View File') . '</a></p><br/>';
                }
            }
		}

		return $customAddress;
	}

	/**
	 * Display extra field value in the WooCommerce shipping address.
	 *
	 * Appends the custom field values to the WooCommerce shipping address. The custom
	 * field values are determined by the 'checkoutify_checkout_data' meta stored in the
	 * order's associated checkout ID. Only custom fields of the 'customField' category
	 * with 'showOnThankYouPage' set to true are considered for display.
	 *
	 * @since 1.0.0
	 *
	 * @param string   $address The current shipping address HTML string.
	 * @param string   $rawAddress The raw shipping address string.
	 * @param WC_Order $order The WooCommerce order object.
	 * @return string The modified shipping address HTML string with custom field values.
	 */
	public function checkoutifyDisplayExtraFieldValueInShipping( $address, $rawAddress, $order ) {
		$address .= $this->addShippingCustomFieldValueInThankyou( $address, $order );
		return $address;
	}

	/**
	 * Add custom shipping field value to the WooCommerce thank you page.
	 *
	 * Retrieves and formats specific custom shipping fields associated with the order
	 * from the related checkout information. The fields are determined by the 'checkoutify_checkout_data'
	 * meta stored in the order's associated checkout ID. Only custom fields of the 'customField'
	 * category with 'showOnThankYouPage' set to true are considered for display.
	 *
	 * @since 1.0.0
	 *
	 * @param string   $address The current shipping address HTML string.
	 * @param WC_Order $order The WooCommerce order object.
	 * @return string The modified shipping address HTML string with custom field values.
	 */
	private function addShippingCustomFieldValueInThankyou( $address, $order ) {
		$checkout_id         = $order->get_meta( 'checkout_id' );
		$checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
		$customAddress       = '';
		$checkoutFields      = $this->formatedInputFields( $checkoutInformation );
		if ( empty( $checkoutFields ) ) {
			return $customAddress;
		}

		foreach ( $checkoutFields as $index => $field ) {
			if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showOnThankYouPage'] ) && $field['showOnThankYouPage'] && isset( $field['section'] ) && 'shipping' === $field['section'] ) {
				$orderMetaData = $order->get_meta( $field['name'] );
				if ( !empty( $orderMetaData ) ) {
					$customAddress .= '<br/>' . $orderMetaData . '<br/>';
				}
			}
            if(!empty($field['fieldType']) && 'file' === $field['fieldType'] && 'shipping' === $field['section']  ) {
                $file_url = $order->get_meta( '_custom_file_field' );
                if ($file_url) {
                    $customAddress .= '<br/><p><strong>' . __('Uploaded File') . ':</strong> <a href="' . esc_url($file_url) . '" target="_blank">' . __('View File') . '</a></p><br/>';
                }
            }
		}

		return $customAddress;
	}

	/**
	 * Display a notice in the WordPress admin area if the license is not activated.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function checkoutifyNotice() {
		$screen = get_current_screen();
		if ( !Functions::maybeLicenseActivate() && ( 'dashboard' === $screen->id || 'plugins' === $screen->id ) ) {
			echo '<div class="notice notice-warning is-dismissible">
					<p>Checkoutify won\'t perform without activating license. Active your license first</p>
				</div>';
		}
	}

	/**
	 * Enqueue wp media uploader.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueueMediaUploaderScripts() {
		if ( is_admin() ) {
			wp_enqueue_media();
		}
	}

	/**
	 * Handles the application of an order bump during the checkout process.
	 *
	 * This function processes the order bump details sent via AJAX, updates the cart accordingly,
	 * and manages the order bump cookie for further actions. It handles scenarios like replacing
	 * the current cart, restoring the previous cart, and adding or removing specific products based on ID of the product.
	 *
	 * @since 1.0.0
	 */
	public function checkoutifyApplyOrderbump() {
		check_ajax_referer( 'checkout_nonce', 'security' );
		$quantity      = isset( $_POST['quantity'] ) ? sanitize_text_field( $_POST['quantity'] ) : 1;
		$productId     = isset( $_POST['productId'] ) ? sanitize_text_field( $_POST['productId'] ) : 0;
		$regularPrice  = isset( $_POST['regularPrice'] ) ? sanitize_text_field( $_POST['regularPrice'] ) : 0;
		$salePrice     = isset( $_POST['salePrice'] ) ? sanitize_text_field( $_POST['salePrice'] ) : 0;
		$discountPrice = isset( $_POST['discountPrice'] ) ? sanitize_text_field( $_POST['discountPrice'] ) : 0;
		$replace       = isset( $_POST['replace'] ) ? sanitize_text_field( $_POST['replace'] ) : '';
		$checked       = isset( $_POST['checked'] ) ? sanitize_text_field( $_POST['checked'] ) : 'no';

		if ( \WC()->cart->is_empty() ) {
			wp_send_json(
				array(
					'success' => false,
					'message' => 'Cart is empty',
				)
			);
		}

		if ( 'yes' === $checked ) {
			if ( 'yes' === $replace ) {
				WC()->cart->empty_cart();
			}
			$this->updateCartPriceWithOrderBump( $productId, $quantity, $discountPrice, $salePrice, $regularPrice );
		} else {
			if ( 'yes' === $replace ) {
				WC()->cart->empty_cart();
				if ( isset( $_COOKIE['checkoutify_order_bump'] ) ) {
					$decodedProducts = json_decode( wp_unslash( $_COOKIE['checkoutify_order_bump'] ), true );
					if ( !empty( $decodedProducts['mainProducts'] ) ) {
						foreach ( $decodedProducts['mainProducts'] as $product ) {
							$mainProductId = !empty( $product['variation_id'] ) ? $product['variation_id'] : $product['product_id'];
							$this->updateCartPriceWithOrderBump( $mainProductId, $product['quantity'], 0, $product['price'], 0, true );
						}
					}
				}
			} else {
				$cartItems = WC()->cart->get_cart();
				if ( !empty( $cartItems ) ) {
					foreach ( $cartItems  as $key =>$item ) {
						$itemId = !empty( $item['variation_id'] ) ? $item['variation_id'] : $item['product_id'];
						if ( $itemId === intval( $productId ) && isset( $item['checkoutify_orderbump_product'] ) ) {
							WC()->cart->remove_cart_item( $item['key'] );
						}
					}
				}
			}
		}

		wp_send_json(
			array(
				'success' => true,
				'message' => 'Apply success',
			)
		);
	}

	/**
	 * Updates the cart price with order bump for a given product.
	 *
	 * @param int   $productId     The product ID.
	 * @param int   $quantity      The quantity of the product to add to the cart.
	 * @param float $discountPrice The discount price for the product.
	 * @param float $salePrice     The sale price for the product.
	 * @param float $regularPrice  The regular price for the product.
	 * @param bool  $isMainProduct Indicates if the product is a main product (default: false).
	 */
	public function updateCartPriceWithOrderBump( $productId, $quantity, $discountPrice, $salePrice, $regularPrice, $isMainProduct = false ) {
		$cart_item_data = array();

		$cookie_name                   = 'wpfnl_order_bump';
		$cookie                        = isset( $_COOKIE[ $cookie_name ] ) ? json_decode( wp_unslash( $_COOKIE[ $cookie_name ] ), true ) : array();
		$cookie['order_bump_accepted'] = 'yes';

		ob_start();
		setcookie( $cookie_name, wp_json_encode( $cookie ), time() + 3600 * 6, '/', COOKIE_DOMAIN );
		ob_end_flush();

		WC()->session->set( 'order_bump_accepted', 'yes' );

		if ( $isMainProduct ) {
			$cart_item_data['line_total'] = $salePrice / $quantity;
		} else {
			if ( isset( $discountPrice ) && 0 < $discountPrice ) {
				$cart_item_data['custom_price'] = $discountPrice / $quantity;
				$cart_item_data['checkoutify_order_bump_custom_price'] = true;
			} elseif ( isset( $salePrice ) && 0 < $salePrice ) {
				$cart_item_data['custom_price'] = $salePrice / $quantity;
				$cart_item_data['checkoutify_order_bump_custom_price'] = true;
			} elseif ( isset( $regularPrice ) && 0 < $regularPrice ) {
				$cart_item_data['custom_price'] = $regularPrice / $quantity;
			}
			$cart_item_data['checkoutify_orderbump_product'] = 'yes';
		}

		$productObject = wc_get_product( $productId );

		$variationId = 0;
        if ( $productObject && is_a( $productObject, 'WC_Product' ) && $productObject->is_type( 'variation' ) ) {
			$variationId = $productObject->get_id();
			$productId   = $productObject->get_parent_id();
		}
		WC()->cart->add_to_cart( $productId, $quantity, $variationId, [], $cart_item_data );

		/**
		 * Fires after an order bump is accepted.
		 *
		 * @param int $order_id The ID of the order.
		 * @param int $product_id The ID of the product associated with the order bump.
		 * @since 1.0.0
		 */
		do_action( 'wpfunnels/order_bump_accepted', get_the_ID(), $productId );
	}

	/**
	 * Sets cookies for order bump items in the cart.
	 *
	 * This function checks if the 'checkoutify_order_bump' cookie is not set and the cart is not empty.
	 * If conditions are met, it extracts relevant data from the cart items and sets the cookie with the data.
	 */
	public function setCartItemCookies() {
		if ( is_admin() || \WC()->cart->is_empty() || $this->maybeOrderbumpClicked() ) {
			return;
		}

		if ( isset( $_REQUEST['wc-ajax'] ) && ( 'apply_coupon' === $_REQUEST['wc-ajax'] || 'wc_stripe_get_cart_details' === $_REQUEST['wc-ajax'] ) ) {
			return;
		}
		global $post;

		if (
			is_admin()
			|| isset( $_GET[ 'removed_item' ] )
			|| !$post
			|| empty( $_POST )
		) {
			return;
		}

		$cartItems   = \WC()->cart->get_cart();
		$cookie_name = 'checkoutify_order_bump';

		if ( isset( $_COOKIE[ $cookie_name ] ) && !empty( $cartItems ) ) {
			$count           = 0;
			$decodedProducts = json_decode( wp_unslash( $_COOKIE[ $cookie_name ] ), true );

			foreach ( $cartItems as $key => $item ) {
				if ( !isset( $item['checkoutify_orderbump_product'] ) ) {
					$count++;
					$quantityCheck = false;
					if ( !empty( $decodedProducts['mainProducts'] ) ) {
						$key           = array_search( $item['product_id'], array_column( $decodedProducts['mainProducts'], 'product_id' ) );
						$cookieProduct = $decodedProducts['mainProducts'][ $key ];
						if ( $item['quantity'] !== $cookieProduct['quantity'] ) {
							$quantityCheck = true;
							break;
						}
					}
				}
			}

			if ( ( !empty( $decodedProducts['mainProducts'] ) && count( $decodedProducts['mainProducts'] ) !== $count ) || ( !empty( $decodedProducts['mainProducts'] ) && $quantityCheck ) ) {
				unset( $_COOKIE[ $cookie_name ] );
				ob_start();
				setcookie( $cookie_name, '', time() - 3600 * 6, '/', COOKIE_DOMAIN );
				ob_end_flush();
			}
		}

		if ( !isset( $_COOKIE[ $cookie_name ] ) && !empty( $cartItems ) ) {
			$cookieData = array();
			$data       = array();
			foreach ( $cartItems as $key => $item ) {
				if ( !isset( $item['checkoutify_orderbump_product'] ) ) {
					$data[] = array(
						'product_id'   => $item['product_id'],
						'variation_id' => $item['variation_id'],
						'quantity'     => $item['quantity'],
						'price'        => $item['line_total'],
					);
				}
			}
			$cookieData['mainProducts'] = $data;
			ob_start();
			setcookie( $cookie_name, wp_json_encode( $cookieData ), time() + 3600 * 6, '/', COOKIE_DOMAIN );
			ob_end_flush();
		} else {
			$cartItems = WC()->cart->get_cart();
			if ( !empty( $cartItems ) ) {
				foreach ( $cartItems  as $key =>$item ) {
					if ( isset( $item['checkoutify_orderbump_product'] ) ) {
						WC()->cart->remove_cart_item( $item['key'] );
					}
				}

				$cartData = \WC()->cart->get_cart();
				if ( isset( $_COOKIE['checkoutify_order_bump'] ) && empty( $cartData ) ) {
					$decodedProducts = json_decode( wp_unslash( $_COOKIE['checkoutify_order_bump'] ), true );
					if ( !empty( $decodedProducts['mainProducts'] ) ) {
						foreach ( $decodedProducts['mainProducts'] as $product ) {
							$mainProductId = !empty( $product['variation_id'] ) ? $product['variation_id'] : $product['product_id'];
							$this->updateCartPriceWithOrderBump( $mainProductId, $product['quantity'], 0, $product['price'], 0, true );
						}
					}
				}
			}
		}
	}


	/**
	 * Check order bump clicked or not from post data
	 *
	 * @return bool True|False
	 * @since 1.0.0
	 */
	public function maybeOrderbumpClicked() {
		if ( !empty( $_POST[ 'post_data' ] ) ) {
			wp_parse_str( $_POST[ 'post_data' ], $post_data );
			return !empty( $post_data[ 'checkoutify_orderbump_clicked' ] ) && 'yes' === $post_data[ 'checkoutify_orderbump_clicked' ];
		}
		return !empty( $_POST[ 'checkoutify_orderbump_clicked' ] ) && 'yes' === $_POST[ 'checkoutify_orderbump_clicked' ];
	}

	/**
	 * Reset the cookie for main products in the checkout process.
	 *
	 * This function unsets and clears the specified cookie associated with order bumps
	 * used during the e-commerce checkout. It ensures that the cookie is removed
	 * both from the $_COOKIE superglobal and the client's browser.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function resetCookieForMainProducts() {
		$cookie_name = 'checkoutify_order_bump';
		if ( isset( $_COOKIE[ $cookie_name ] ) ) {
			unset( $_COOKIE[ $cookie_name ] );
			ob_start();
			setcookie( $cookie_name, '', time() - 3600 * 6, '/', COOKIE_DOMAIN );
			ob_end_flush();
		}
	}

	/**
	 * Remove default checkout validation
	 *
	 * @param array  $data
	 * @param object $errors
	 * @return void
	 * @since 1.0.0
	 */
	public function removeDefaultCheckoutValidation( $data, $errors ) {
		$checkoutId = !empty( $_POST['checkoutify_id'] ) ? $_POST['checkoutify_id'] : '';

		if ( !$checkoutId ) {
			global $post;
			$stepId     = isset( $post->ID ) ? $post->ID : '';
			$funnelId   = get_post_meta( $stepId, '_funnel_id', true );
			$checkoutId = Functions::maybeCheckoutAvailableByFunnelId( $funnelId );
		}

		// Check if the custom shortcode is present.
		$custom_shortcode_present = Functions::isCheckoutifyShortcodeAvailable(); // Replace with your actual check.
		$defaultCheckout          = Functions::getDefaultCheckout();
		if ( ( ( $custom_shortcode_present || $defaultCheckout ) || ( $checkoutId ) ) ) {
			// Check if 'Terms and Conditions' is required and not checked

			if ( isset( $_POST['terms-field'] ) ) {
				if ( !isset( $_POST['terms'] ) || ( isset( $_POST['terms'] ) && $_POST['terms'] != '1' ) ) {
					$errors->errors         = array();
					$errors->error_messages = array();
					$errors->error_data     = array();
					$errors->add( 'terms_error', __( 'Please read and accept the terms and conditions to proceed with your order.', 'woocommerce' ) );
				} else {
					$errors->errors         = array();
					$errors->error_messages = array();
					$errors->error_data     = array();
				}
			} else {
				$errors->errors         = array();
				$errors->error_messages = array();
				$errors->error_data     = array();
			}
		}
	}

    /**
     * Display custom field in the WooCommerce thank you page.
     *
     * This function retrieves and displays specific custom fields associated with the order
     * from the related checkout information. The fields are determined by the 'checkoutify_checkout_data'
     * meta stored in the order's associated checkout ID.
     *
     * @since 1.0.7
     *
     * @param int $order_id The ID of the WooCommerce order.
     * @return void
     */
    public function display_custom_field_in_thankyou_page( $order ) {
        $checkout_id = $order->get_meta( 'checkout_id' );
        $checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
        $checkoutFields = $this->formatedInputFields( $checkoutInformation );
        if ( empty( $checkoutFields ) ) {
            return;
        }
        foreach ( $checkoutFields as $index => $field ) {
            if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showOnThankYouPage'] ) && $field['showOnThankYouPage'] && isset( $field['section'] ) && 'other' === $field['section'] ) {
                $orderMetaData = $order->get_meta( $field['name'] );
                if ( !empty( $orderMetaData ) ) {
                    echo '<br/>';
                    echo '<p><strong>' . __( $field['label'] ) . ':</strong> ' . $orderMetaData . '</p>';
                }
            }
        }

    }

    /**
     * Display custom field in the WooCommerce order details on the admin page.
     *
     * This function retrieves and displays specific custom fields associated with the order
     * from the related checkout information. The fields are determined by the 'checkoutify_checkout_data'
     * meta stored in the order's associated checkout ID.
     *
     * @since 1.0.7
     *
     * @param WC_Order $order The WooCommerce order object.
     * @return void
     */
    public function display_custom_field_in_order_details_page( $order ){
        $checkout_id = $order->get_meta( 'checkout_id' );
        $checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
        $checkoutFields = $this->formatedInputFields( $checkoutInformation );
        if ( empty( $checkoutFields ) ) {
            return;
        }
        foreach ( $checkoutFields as $index => $field ) {
            if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showInOrderPage'] ) && $field['showInOrderPage'] && isset( $field['section'] ) && 'other' === $field['section'] ) {
                $orderMetaData = $order->get_meta( $field['name'] );
                if ( !empty( $orderMetaData ) ) {
                    echo '<br/>';
                    echo '<p><strong>' . __( $field['label'] ) . ':</strong> ' . $orderMetaData . '</p>';
                }
            }
        }
    }

    /**
     * Display custom field in the WooCommerce email page.
     *
     * This function retrieves and displays specific custom fields associated with the order
     * from the related checkout information. The fields are determined by the 'checkoutify_checkout_data'
     * meta stored in the order's associated checkout ID.
     *
     * @since 1.0.7
     *
     * @param WC_Order $order The WooCommerce order object.
     * @param bool $sent_to_admin
     * @param bool $plain_text
     * @param WC_Email $email
     * @return void
     */
    public function display_custom_field_in_email_page($order, $sent_to_admin, $plain_text, $email) {
        $checkout_id = $order->get_meta( 'checkout_id' );
        $checkoutInformation = get_post_meta( $checkout_id, Keys::CHECKOUTIFY_CHECKOUT_META, true );
        $checkoutFields = $this->formatedInputFields( $checkoutInformation );
        if ( empty( $checkoutFields ) ) {
            return;
        }
        foreach ( $checkoutFields as $index => $field ) {
            if ( isset( $field['fieldCategory'] ) && 'customField' === $field['fieldCategory'] && isset( $field['showInEmail'] ) && $field['showInEmail'] && isset( $field['section'] ) && 'other' === $field['section'] ) {
                $orderMetaData = $order->get_meta( $field['name'] );
                if ( !empty( $orderMetaData ) ) {
                    echo '<br/>';
                    echo '<p><strong>' . __( $field['label'] ) . ':</strong> ' . $orderMetaData . '</p>';
                }
            }
        }
    }

    /**
     * Removes a coupon from the WooCommerce cart via an AJAX request.
     *
     * This function checks for the presence of a valid nonce, verifies that a coupon code
     * is provided, and ensures the coupon is applied to the cart before attempting to remove it.
     * The response is sent in JSON format indicating success or failure.
     *
     * @since 1.0.9
     *
     * @return void
     */
    public function checkoutifyRemoveCoupon() {
        check_ajax_referer( 'checkout_nonce', 'security' );
        if ( ! WC()->cart ) {
            wp_send_json(
                array(
                    'success' => false,
                    'message' => ___('Cart is empty','checkoutify'),
                )
            );
        }

        if ( ! isset( $_POST['code'] ) ) {
            wp_send_json(
                array(
                    'success' => false,
                    'message' => __('Coupon code is missing','checkoutify'),
                )
            );
        }

        $code = sanitize_text_field( $_POST['code'] );

        if ( ! WC()->cart->has_discount( $code ) ) {
            wp_send_json(
                array(
                    'success' => false,
                    'message' => __('Coupon not found in the cart', 'checkoutify'),
                )
            );
        }

        WC()->cart->remove_coupon( $code );

        wp_send_json(
            array(
                'success' => true,
                'message' => __('Coupon removed successfully', 'checkoutify'),
            )
        );
    }

}
