<?php

namespace RexTheme\Checkoutify\Rest\Actions;

use RexTheme\Checkoutify\Common\Keys;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class RexLicenseManager {

	/**
	 * Validates and processes a license key using a custom API endpoint.
	 *
	 * This method sends a request to a custom API endpoint for license validation and processing.
	 * It includes the provided license key, action, item ID, and site URL in the API request.
	 * The response from the API is then decoded, and the appropriate message and status are determined based on the response data.
	 * The license status, data, and premium status are updated in the options based on the API response.
	 *
	 * @param string $action  The action to perform on the license key (e.g., 'activate' or 'deactivate').
	 * @param string $license The license key to be validated or processed.
	 *
	 * @return array An associative array containing the status and message of the license validation or processing.
	 *               The 'status' key can have values 'Success' or 'Alert'.
	 *               The 'message' key contains the detailed message corresponding to the license status.
	 *
	 * @since 1.0.0
	 */
	public static function validate_license( $action, $license ) {
        if ( !defined( 'REX_CHECKOUTIFY_ITEM_ID' ) || !defined( 'REX_CHECKOUTIFY_SL_STORE_URL' ) ) {
            return [
                'status'  => 'Alert',
                'message' => __( 'An unauthorized action initiated.', 'checkoutify' )
            ];
        }

		$api_params = [
			'edd_action' => "{$action}_license",
			'license'    => $license,
			'item_id'    => REX_CHECKOUTIFY_ITEM_ID, // The ID of the item in RexTheme
			'url'        => home_url()
		];

		// Call the custom API.
		$response = wp_remote_post( REX_CHECKOUTIFY_SL_STORE_URL, [ 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ] );

		// make sure the response came back okay
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
			$message = ( is_wp_error( $response ) && !empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.', 'checkoutify' );
            $status = 'deactivate';
		}
		else {
			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			if ( false === $license_data->success ) {
				switch( $license_data->error ) {
					case 'expired' :
						$message = sprintf(
							__( 'Your license key expired on %s.', 'checkoutify' ),
							date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
						);
						break;
					case 'revoked' :
						$message = __( 'Your license key has been disabled.', 'checkoutify' );
						break;
					case 'missing' :
						$message = __( 'Invalid license.', 'checkoutify' );
						break;
					case 'invalid' :
					case 'site_inactive' :
						$message = __( 'Your license is not active for this URL.', 'checkoutify' );
						break;
					case 'item_name_mismatch' :
						$message = sprintf( __( 'This appears to be an invalid license key for %s.', 'checkoutify' ), 'Product Recommendation for WooCommerce' );
						break;
					case 'no_activations_left':
						$message = __( 'Your license key has reached its activation limit.', 'checkoutify' );
						break;
					case 'deactivated':
						$message = __( 'Your license successfully deactivate.', 'checkoutify' );
						break;
					case 'failed':
						$message = __( 'Your license deactivation failed.', 'checkoutify' );
						break;
					default:
						$message = __( 'An unknown error occurred.', 'checkoutify' );
						break;
				}
			}
			else {
				update_option( Keys::CHECKOUTIFY_LICENSE_KEY, trim( $license ) );
			}
            $status = 'valid' === $license_data->license ? 'activate' : 'deactivate';
			update_option( Keys::CHECKOUTIFY_LICENSE_STATUS, $status );
			update_option( Keys::CHECKOUTIFY_LICENSE_DATA, $license_data );
		}

		if ( empty( $message ) ) {
			switch( $action ) {
				case 'activate':
					$message = __( 'Your license has been successfully activated.', 'checkoutify' );
					break;
				case 'deactivate':
					$message = __( 'Your license has been successfully deactivated.', 'checkoutify' );
					break;
				case 'check':
					$message = __( 'Your license has been successfully validated.', 'checkoutify' );
					break;
				default:
					$message = __( 'An unauthorized action has been initiated.', 'checkoutify' );
					break;
			}
		}

        return [
            'status'   => $status,
            'message'  => $message,
            'success'  => 'activate' === $status,
            'isActive' => 'activate' === $status,
        ];
	}
}
