<?php
/**
 * License Controller
 *
 * @package RexTheme\Checkoutify\REST
 * @version 1.0.0
 * @author RexTheme
 */

namespace RexTheme\Checkoutify\REST;

use RexTheme\Checkoutify\Abstracts\RESTController;
use RexTheme\Checkoutify\Common\Keys;
use RexTheme\Checkoutify\Rests\Actions\Admin\RexLicenseManager;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

/**
 * API layoutController class.
 *
 * @since 0.3.0
 */
class LicenseController extends RESTController {

    /**
     * Route base.
     *
     * @var string
     */
    protected $base = 'license';


    /**
     * Register all routes related to License
     *
     * @throws \Exception
     */
    public function registerRoutes() {

        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/get',
            array(
                array(
                    'methods'             => WP_REST_Server::READABLE,
                    'callback'            => array( $this, 'getLicense' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );


        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/activate',
            array(
                array(
                    'methods'             => WP_REST_Server::EDITABLE,
                    'callback'            => array( $this, 'activateLicense' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'args'                => array(
                        'licenseKey' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                    ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );


        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/deactivate',
            array(
                array(
                    'methods'             => WP_REST_Server::EDITABLE,
                    'callback'            => array( $this, 'deActiveLicense' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'args'                => array(
                        'licenseKey' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                    ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );
    }


    /**
     * Get license
     *
     * @return WP_REST_Response|null
     *
     * @since 1.0.0
     */
    public function getLicense(): ? WP_REST_Response {
        $license_details    = $this->getLicenseDetails();
        return rest_ensure_response( $license_details );
    }


    /**
     * Activate license
     *
     * @param WP_REST_Request $request
     * @return WP_REST_Response|null
     * @throws \Exception
     */
    public function activateLicense( \WP_REST_Request $request ): ?WP_REST_Response {
        if( !$request->has_param('licenseKey') ){
            return rest_ensure_response(
                $this->prepareWpErrorResponse(
                    __( "License key is require for activation process.", 'checkoutify' ), //phpcs:ignoreFile
                    400
                )
            );
        }
        $license_key = $request->get_param( 'licenseKey' );
        if ( defined('CHECKOUTIFY_AUTHOR_REX') &&  class_exists( '\RexTheme\Checkoutify\Rest\Actions\RexLicenseManager' ) && method_exists( '\RexTheme\Checkoutify\Rest\Actions\RexLicenseManager', 'validate_license' ) ) {
            $responseData = \RexTheme\Checkoutify\Rest\Actions\RexLicenseManager::validate_license( 'activate', $license_key );
        }
        else {
            $license_response = $this->check_license( $license_key );
            $responseData     = $this->operateLicense( $license_key, $license_response );
        }
        return rest_ensure_response( $responseData );
    }


    /**
     * Deactivate license
     *
     * @param WP_REST_Request $request
     * @return WP_REST_Response|null
     * @throws \Exception
     *
     * @since 1.0.0
     */
    public function deActiveLicense(\WP_REST_Request $request ): ?WP_REST_Response {
        if( !$request->has_param('licenseKey') ){
            return rest_ensure_response(
                $this->prepareWpErrorResponse(
                    __( "License key is require for activation process.", 'checkoutify' ), //phpcs:ignoreFile
                    400
                )
            );
        }
        $license_key = $request->get_param( 'licenseKey' );
        if ( defined('CHECKOUTIFY_AUTHOR_REX') && class_exists( '\RexTheme\Checkoutify\Rest\Actions\RexLicenseManager' ) && method_exists( '\RexTheme\Checkoutify\Rest\Actions\RexLicenseManager', 'validate_license' ) ) {
            $responseData = \RexTheme\Checkoutify\Rest\Actions\RexLicenseManager::validate_license( 'deactivate', $license_key );
        }
        else {
            $license_response = $this->check_license( $license_key, 'deactivate' );
            $responseData     = $this->operateLicense( $license_key, $license_response );
        }
        return rest_ensure_response( $responseData );
    }


    /**
     * Operate license
     *
     * @param $license_key
     * @param string $action
     * @return mixed|string|void
     * @throws \Exception
     */
    private function operateLicense( $license_key, $license_response ) {
        $data = array(
            'key'           => '',
            'last_check'    => time(),
            'start_date'    => '',
            'end_date'      => '',
        );

        $responseData = array(
            'success'   => true,
            'message'   => __( 'License successfully activate on your site', 'checkoutify' ),
            'status'    => 'success',
            'isActive'  => true
        );

        $license_data = isset($license_response[0]) ? $license_response[0] : '';
        update_option( Keys::CHECKOUTIFY_LICENSE_KEY, $license_key );
        if( is_object($license_data) ) {
            if($license_data->status_code == 's100' || $license_data->status_code == 's101') {
                $message        = $data->message;
                $license_start  = isset($data->licence_start) ? $data->licence_start: '';
                $license_end    = isset($data->licence_expire) ? $data->licence_expire : '';

                //save the license
                $data = array(
                    'key'           => $license_key,
                    'last_check'    => time(),
                    'start_date'    => $license_start,
                    'end_date'      => $license_end,
                );
                update_option( Keys::CHECKOUTIFY_LICENSE_STATUS, 'activate' );
                update_option( Keys::CHECKOUTIFY_LICENSE_DATA, $data );
                $responseData = array(
                    'success'   => true,
                    'message'   => __( 'License successfully activate on your site', 'checkoutify' ),
                    'status'    => 'activate',
                    'isActive'  => true
                );

            }elseif ( $license_data->status_code == 'e002' || $license_data->status_code == 'e104' || $license_data->status_code == 'e211' ) {
                $message            = $license_data->message;
           
                update_option(Keys::CHECKOUTIFY_LICENSE_DATA, $data );
                update_option( Keys::CHECKOUTIFY_LICENSE_STATUS, 'deactivate' );
                $responseData = array(
                    'success'   => false,
                    'message'   => $message,
                    'status'    => 'deactivate',
                    'isActive'  => false
                );
            }else{
                $message        = $license_data->message;
               
                update_option(Keys::CHECKOUTIFY_LICENSE_DATA, $data );
                update_option( Keys::CHECKOUTIFY_LICENSE_STATUS, 'deactivate' );
                $responseData = array(
                    'success'   => false,
                    'message'   => $message,
                    'status'    => 'deactivate',
                    'isActive'  => false
                );
            }
        }
        return $responseData;
    }



    /**
     * Check license
     *
     * @param $license_key
     * @param string $action
     * @return mixed|string|void
     * @throws \Exception
     */
    private function check_license( $license_key, $action = 'activate' ) {
        $api_params = array(
            'woo_sl_action'     => $action,
            'licence_key'       => $license_key,
            'product_unique_id' => CHECKOUTIFY_ITEM_ID,
            'domain'            => CHECKOUTIFY_INSTANCE,
            'license_url'       => CHECKOUTIFY_LICENSE_URL
        );

        //check if license url is working
        $_status_data = wp_remote_get( CHECKOUTIFY_LICENSE_URL );

        if( is_wp_error( $_status_data ) || $_status_data['response']['code'] != 200 ) {
            $license_url    = CHECKOUTIFY_LICENSE_MIDDLEMAN_URL . '?' . http_build_query( $api_params ) ;
        } else {
            $license_url    = CHECKOUTIFY_LICENSE_URL . '?' . http_build_query( $api_params );
        }

        $response = wp_remote_get(
            $license_url,
            array(
                'timeout' => 15, 'sslverify' => false
            )
        );
       
        if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
            return ( is_wp_error( $response ) && ! empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.', 'checkoutify' );
        }else {
            return json_decode( wp_remote_retrieve_body( $response ) );
        }
    }


    /**
     * Get License information
     *
     * @return array
     */
    private function getLicenseDetails() {
        return array(
            'key'       => get_option(Keys::CHECKOUTIFY_LICENSE_KEY),
            'status'    => get_option(Keys::CHECKOUTIFY_LICENSE_STATUS, 'deactivate'),
        );
    }
}
