<?php
/**
 * Orderbump Controller
 *
 * @package RexTheme\Checkoutify\REST
 * @version 1.0.0
 * @author RexTheme
 */

namespace RexTheme\Checkoutify\REST;

use RexTheme\Checkoutify\Abstracts\RESTController;
use RexTheme\Checkoutify\Module\OrderBump;
use RexTheme\Checkoutify\Common\Keys;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

/**
 * API OrderbumpController class.
 *
 * @since 0.3.0
 */
class OrderbumpController extends RESTController {

    /**
     * Route base.
     *
     * @var string
     */
    protected $base = 'orderbump';


    /**
     * Register all routes related to License
     *
     * @throws \Exception
     */
    public function registerRoutes() {

        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/product/search/(?P<searchItem>\w+)',
            array(
                array(
                    'methods'             => WP_REST_Server::EDITABLE,
                    'callback'            => array( $this, 'searchProduct' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );

        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/create',
            array(
                array(
                    'methods'             => WP_REST_Server::EDITABLE,
                    'callback'            => array( $this, 'createOrderbump' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'args'                => array(
                        'id' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                    ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );
        
        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/update',
            array(
                array(
                    'methods'             => WP_REST_Server::EDITABLE,
                    'callback'            => array( $this, 'updateOrderbump' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'args'                => array(
                        'id' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                        'index' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                    ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );
        
        register_rest_route(
            $this->namespace,
            '/' . $this->base . '/delete',
            array(
                array(
                    'methods'             => WP_REST_Server::EDITABLE,
                    'callback'            => array( $this, 'deleteOrderbump' ),
                    'permission_callback' => array( $this, 'checkPermission' ),
                    'args'                => array(
                        'id' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                        'index' => array(
                            'type'        => 'string',
                            'required'    => true,
                            'sanitize_callback' => function ( $param ) {
                                return sanitize_text_field( $param );
                            },
                        ),
                    ),
                    'schema'              => array( $this, 'getItemSchema' ),
                ),
            )
        );
    }


    /**
     * Search product for orderbump
     *
     * @param WP_REST_Request $request
     * @return WP_REST_Response|null
     * @throws \Exception
     *
     * @since 1.0.0
     */
    public function searchProduct(\WP_REST_Request $request ): ?WP_REST_Response {
        if( !$request->has_param('searchItem') ){
            return rest_ensure_response(
                $this->prepareWpErrorResponse(
                    __( "Search Item is require for getting products.", 'checkoutify' ), //phpcs:ignoreFile
                    400
                )
            );
        }
        $term    = $request->get_param('searchItem');
        $obInstance = new OrderBump();
        $products = $obInstance->getObProduct( $term );
        return rest_ensure_response( $products );
    }


    /**
     * Create Orderbump
     * 
     * @param WP_REST_Request $request
     * @return WP_REST_Response|null
     * @throws \Exception
     *
     * @since 1.0.0
     */
    public function createOrderbump(\WP_REST_Request $request ): ?WP_REST_Response {
        if( !$request->has_param('id', 'data') ){
            return rest_ensure_response(
                $this->prepareWpErrorResponse(
                    __( "Checkout ID and data are required for creating orderbump.", 'checkoutify' ), //phpcs:ignoreFile
                    400
                )
            );
        }
        $checkoutId    = $request->get_param('id');
        $data    = $request->get_param('data');
        $obInstance = new OrderBump();
        $response = $obInstance->createOrderBump( $checkoutId, $data );
        return rest_ensure_response( $response );
    }
    
    
    /**
     * Update Orderbump
     * 
     * @param WP_REST_Request $request
     * @return WP_REST_Response|null
     * @throws \Exception
     *
     * @since 1.0.0
     */
    public function updateOrderbump(\WP_REST_Request $request ): ?WP_REST_Response {
        if( !$request->has_param('id', 'data', 'index') ){
            return rest_ensure_response(
                $this->prepareWpErrorResponse(
                    __( "Checkout ID, data and Index are required for creating orderbump.", 'checkoutify' ), //phpcs:ignoreFile
                    400
                )
            );
        }
        $checkoutId    = $request->get_param('id');
        $data    = $request->get_param('data');
        $index    = $request->get_param('index');
        $obInstance = new OrderBump();
        $response = $obInstance->updateOrderBump( $checkoutId, $data, $index);
        return rest_ensure_response( $response );
    }
    
    
    /**
     * Update Orderbump
     * 
     * @param WP_REST_Request $request
     * @return WP_REST_Response|null
     * @throws \Exception
     *
     * @since 1.0.0
     */
    public function deleteOrderbump(\WP_REST_Request $request ): ?WP_REST_Response {
        if( !$request->has_param('id', 'index') ){
            return rest_ensure_response(
                $this->prepareWpErrorResponse(
                    __( "Checkout ID, data and Index are required for creating orderbump.", 'checkoutify' ), //phpcs:ignoreFile
                    400
                )
            );
        }
        $checkoutId    = $request->get_param('id');
        $index    = $request->get_param('index');
        $obInstance = new OrderBump();
        $response = $obInstance->deleteOrderbump( $checkoutId, $index);
        return rest_ensure_response( $response );
    }

}
