<?php
/**
 * Elementor Manager class
 *
 * @package RexTheme\Checkoutify\Widgets
 */

namespace RexTheme\Checkoutify\Widgets\Elementor;

use RexTheme\Checkoutify\Common\Functions;
use RexTheme\Checkoutify\Common\Keys;
use RexTheme\Checkoutify\Widgets\Elementor\CheckoutWidget;
/**
 * Class Manager
 *
 * @package RexTheme\Checkoutify\Widgets
 */
final class Manager {


	/**
	 * Instance
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @static
	 *
	 * @var Manager The single instance of the class.
	 */
	private static $instance = null;

	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @return Manager An instance of the class.
	 * @since  1.0.0
	 *
	 * @access public
	 * @static
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function __construct() {
		if ( $this->maybeCompatible() ) {
			add_action( 'elementor/init', array( $this, 'init' ) );
			add_action( 'elementor/editor/after_enqueue_styles', array( $this, 'enqueueElementorCustomStyle' ) );
		}
	}


	/**
	 * Add css file on  Elementor admin
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function enqueueElementorCustomStyle() {
		wp_enqueue_style( 'checkoutify-elementor-icon', CHECKOUTIFY_URL . '/includes/Widgets/Elementor/Assets/Css/ElemetorStyle.css' );
	}


	 /**
	  * Initialize the plugin
	  *
	  * Load the plugin only after Elementor (and other plugins) are loaded.
	  * Load the files required to run the plugin.
	  *
	  * Fired by `plugins_loaded` action hook.
	  *
	  * @since 1.0.0
	  *
	  * @access public
	  */
	public function init() {
		add_action( 'elementor/init', array( $this, 'addCategory' ), 9999 );
		if ( version_compare( ELEMENTOR_VERSION, '3.5.0', '>=' ) ) {
			add_action( 'elementor/widgets/register', array( $this, 'initWidgets' ) );
		} else {
			add_action( 'elementor/widgets/widgets_registered', array( $this, 'initWidgets' ) );
		}
	}


	/**
	 * Register Category
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	public function addCategory() {
		$elementsManager = \Elementor\Plugin::instance()->elements_manager;
		$elementsManager->add_category(
			'checkoutify',
			array(
				'title' => __( 'Checkoutify', 'checkoutify' ),
				'icon'  => 'fa fa-plug',
			)
		);
	}


	/**
	 * Init Widgets
	 *
	 * Include widgets files and register them
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function initWidgets() {
		$widgetInstance = new CheckoutWidget();
		if ( version_compare( ELEMENTOR_VERSION, '3.5.0', '>=' ) ) {
			\Elementor\Plugin::instance()->widgets_manager->register( $widgetInstance );
		} else {
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type( $widgetInstance );
		}
	}


	/**
	 * Compatibility Checks
	 *
	 * Checks if the installed version of Elementor meets the plugin's minimum requirement.
	 * Checks if the installed PHP version meets the plugin's minimum requirement.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function maybeCompatible() {
		// Check if Elementor installed and activated
		if ( !did_action( 'elementor/loaded' ) ) {
			return false;
		}
		// Check for required Elementor version
		if ( !version_compare( ELEMENTOR_VERSION, Keys::CHECKOUTIFY_MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
			return false;
		}
		// Check for required PHP version
		if ( version_compare( PHP_VERSION, Keys::CHECKOUTIFY_MINIMUM_PHP_VERSION, '<' ) ) {
			return false;
		}
		return true;
	}
}

Manager::instance();
